@@ -2,29 +2,35 @@
 // pandora: syd's Dump Inspector & Profile Writer
 // pandora.rs: Main entry point
 //
-// Copyright (c) 2021, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2021, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
+#[cfg(all(feature = "rust-dns", feature = "system-dns"))]
+compile_error!("features `rust-dns` and `system-dns` are mutually exclusive");
+
+#[cfg(feature = "rust-dns")]
+use std::convert::TryFrom;
+#[cfg(feature = "rust-dns")]
+use std::net::{SocketAddr, SocketAddrV4, SocketAddrV6, UdpSocket};
 use std::{
     borrow::Cow,
     cmp::Ordering,
     env,
-    ffi::{CStr, OsString},
+    ffi::OsString,
     fmt,
     fmt::Write as FmtWrite,
     fs::{metadata, File, OpenOptions},
     hash::{Hash, Hasher},
     io::{self, stderr, stdin, BufRead, BufReader, Read, Write as IoWrite},
     iter::FromIterator,
-    net::{IpAddr, SocketAddrV4, SocketAddrV6},
+    net::IpAddr,
     os::{
         fd::{AsFd, AsRawFd},
         unix::ffi::{OsStrExt, OsStringExt},
     },
     path::{Path, PathBuf},
     process::{exit, Command, ExitCode},
-    ptr,
     str::FromStr,
     sync::{Arc, Mutex},
     thread,
@@ -36,11 +42,7 @@ use console::style;
 use crc::{Crc, CRC_32_ISO_HDLC, CRC_64_ECMA_182};
 use data_encoding::{HEXLOWER, HEXLOWER_PERMISSIVE};
 use indicatif::{MultiProgress, ProgressBar, ProgressStyle};
-use libc::{
-    getnameinfo, pid_t, res_init, socklen_t, EAI_AGAIN, EAI_BADFLAGS, EAI_FAIL, EAI_FAMILY,
-    EAI_MEMORY, EAI_NONAME, EAI_SERVICE, EAI_SOCKTYPE, EAI_SYSTEM, NI_MAXHOST, NI_NAMEREQD,
-    NI_NUMERICSERV,
-};
+use libc::pid_t;
 use memchr::arch::all::is_equal;
 use nix::{
     errno::Errno,
@@ -48,7 +50,6 @@ use nix::{
     sys::{
         signal::{kill, sigprocmask, SigmaskHow, Signal},
         signalfd::SigSet,
-        socket::{SockaddrLike, SockaddrStorage},
         stat::Mode,
     },
     unistd::{pipe2, Pid},
@@ -120,12 +121,12 @@ const CAP_ORDER: &[&str] = &[
     "cnet",
     "snet",
     // base capabilities
+    "fs",
     "walk",
     "stat",
     "read",
     "write",
     "exec",
-    "ioctl",
     "create",
     "delete",
     "rename",
@@ -264,14 +265,74 @@ impl<'de> Deserialize<'de> for Capability {
     }
 }
 
+#[derive(Clone, Debug)]
+enum IoctlEntry {
+    Name(String),
+    Val(u64),
+}
+
+impl<'de> Deserialize<'de> for IoctlEntry {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        struct IoctlEntryVisitor;
+
+        impl<'de> Visitor<'de> for IoctlEntryVisitor {
+            type Value = IoctlEntry;
+
+            fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+                formatter.write_str("integer or string")
+            }
+
+            fn visit_u64<E>(self, value: u64) -> Result<Self::Value, E>
+            where
+                E: serde::de::Error,
+            {
+                Ok(IoctlEntry::Val(value))
+            }
+
+            fn visit_i64<E>(self, value: i64) -> Result<Self::Value, E>
+            where
+                E: serde::de::Error,
+            {
+                if value < 0 {
+                    Err(E::custom("negative integer for ioctl"))
+                } else {
+                    Ok(IoctlEntry::Val(value as u64))
+                }
+            }
+
+            fn visit_str<E>(self, value: &str) -> Result<Self::Value, E>
+            where
+                E: serde::de::Error,
+            {
+                Ok(IoctlEntry::Name(value.to_owned()))
+            }
+
+            fn visit_string<E>(self, value: String) -> Result<Self::Value, E>
+            where
+                E: serde::de::Error,
+            {
+                Ok(IoctlEntry::Name(value))
+            }
+        }
+
+        deserializer.deserialize_any(IoctlEntryVisitor)
+    }
+}
+
 #[derive(Clone, Debug)]
 enum Access {
     Path {
         ctx: String,
         cap: Option<Capability>,
-        ctl: Option<Vec<String>>,
+        fs: Option<String>,
         path: String,
-        args: Option<Vec<u64>>,
+    },
+    Ioctl {
+        ctx: String,
+        ctl: Vec<IoctlEntry>,
     },
     InetAddr {
         ctx: String,
@@ -323,7 +384,8 @@ impl<'de> Deserialize<'de> for Access {
                 let mut addr: Option<String> = None;
                 let mut unix_: Option<String> = None;
 
-                let mut ctl: Option<Vec<String>> = None;
+                let mut ctl: Option<Vec<IoctlEntry>> = None;
+                let mut fs: Option<String> = None;
                 let mut cmd: Option<String> = None;
                 let mut argv: Option<Vec<String>> = None;
                 let mut time_: Option<String> = None;
@@ -376,6 +438,12 @@ impl<'de> Deserialize<'de> for Access {
                             }
                             ctl = map.next_value()?;
                         }
+                        "fs" => {
+                            if fs.is_some() {
+                                return Err(serde::de::Error::duplicate_field("fs"));
+                            }
+                            fs = map.next_value()?;
+                        }
                         "cmd" => {
                             if cmd.is_some() {
                                 return Err(serde::de::Error::duplicate_field("cmd"));
@@ -421,13 +489,9 @@ impl<'de> Deserialize<'de> for Access {
 
                 // Now decide which variant to build based on which fields we have:
                 if let Some(path) = path {
-                    Ok(Access::Path {
-                        ctx,
-                        cap,
-                        ctl,
-                        path,
-                        args,
-                    })
+                    Ok(Access::Path { ctx, cap, fs, path })
+                } else if let Some(ctl) = ctl {
+                    Ok(Access::Ioctl { ctx, ctl })
                 } else if let Some(addr) = addr {
                     Ok(Access::InetAddr { ctx, cap, addr })
                 } else if let Some(unix) = unix_ {
@@ -449,59 +513,106 @@ impl<'de> Deserialize<'de> for Access {
     }
 }
 
-// lookup_addr is borrowed from Syd to avoid depending on Syd.
-// syd::dns::lookup_addr:
-// Performs a reverse DNS lookup for the given IP address, returning a hostname or an error.
+fn io_to_errno(e: std::io::Error) -> Errno {
+    e.raw_os_error().map(Errno::from_raw).unwrap_or(Errno::EIO)
+}
+
+// Performs a reverse DNS lookup for the given IP address,
+// returning a hostname or an error.
 #[expect(clippy::cast_possible_truncation)]
 fn lookup_addr(addr: IpAddr) -> Result<String, Errno> {
-    let addr = match addr {
-        IpAddr::V4(v4) => SockaddrStorage::from(SocketAddrV4::new(v4, 0)),
-        IpAddr::V6(v6) => SockaddrStorage::from(SocketAddrV6::new(v6, 0, 0, 0)),
-    };
-    let mut host_buf = [0i8; NI_MAXHOST as usize];
-
-    // SAFETY: Initialize system DNS resolver.
-    if unsafe { res_init() } != 0 {
-        return Err(Errno::EFAULT);
+    #[cfg(feature = "system-dns")]
+    {
+        dns_lookup::lookup_addr(&addr).map_err(io_to_errno)
     }
 
-    // SAFETY: We call a system function (getnameinfo) with valid pointers for the address
-    // and buffer, and we check the return value to ensure success before using `host_buf`.
-    let ret = unsafe {
-        getnameinfo(
-            addr.as_ptr(),
-            addr.len(),
-            host_buf.as_mut_ptr().cast(),
-            host_buf.len() as socklen_t,
-            ptr::null_mut(),
-            0,
-            NI_NAMEREQD | NI_NUMERICSERV,
-        )
-    };
+    #[cfg(feature = "rust-dns")]
+    {
+        // Read system DNS configuration (max 4KB).
+        let f = File::open("/etc/resolv.conf").map_err(io_to_errno)?;
+        let mut buf = Vec::with_capacity(4096);
+        f.take(4096).read_to_end(&mut buf).map_err(io_to_errno)?;
+        let conf = resolv_conf::Config::parse(&buf).map_err(|_| Errno::EINVAL)?;
 
-    if ret != 0 {
-        if ret == EAI_SYSTEM {
-            return Err(Errno::last());
-        } else {
-            let e = match ret {
-                EAI_AGAIN => Errno::EAGAIN,
-                EAI_BADFLAGS => Errno::EINVAL,
-                EAI_FAIL => Errno::EIO,
-                EAI_FAMILY => Errno::EAFNOSUPPORT,
-                EAI_MEMORY => Errno::ENOMEM,
-                EAI_NONAME => Errno::ENOENT,
-                EAI_SERVICE => Errno::EPROTONOSUPPORT,
-                EAI_SOCKTYPE => Errno::ESOCKTNOSUPPORT,
-                _ => Errno::EIO,
-            };
-            return Err(e);
+        // Pick the first nameserver (IPv4/IPv6).
+        let ns = conf
+            .nameservers
+            .iter()
+            .find_map(|ns| -> Option<SocketAddr> {
+                match ns {
+                    resolv_conf::ScopedIp::V4(ipv4) => {
+                        Some(SocketAddr::V4(SocketAddrV4::new(*ipv4, 53)))
+                    }
+                    resolv_conf::ScopedIp::V6(ipv6, _scope) => {
+                        Some(SocketAddr::V6(SocketAddrV6::new(*ipv6, 53, 0, 0)))
+                    }
+                }
+            })
+            .ok_or(Errno::ENOENT)?;
+
+        // Construct the reverse pointer name.
+        let ptr_name = match addr {
+            IpAddr::V4(ip) => {
+                let octets = ip.octets();
+                format!(
+                    "{}.{}.{}.{}.in-addr.arpa",
+                    octets[3], octets[2], octets[1], octets[0]
+                )
+            }
+            IpAddr::V6(ip) => {
+                let octets = ip.octets();
+                let mut s = String::with_capacity(72);
+                use std::fmt::Write;
+                for octet in octets.iter().rev() {
+                    let _ = write!(s, "{:x}.{:x}.", octet & 0x0f, (octet >> 4) & 0x0f);
+                }
+                s + "ip6.arpa"
+            }
+        };
+
+        // Send Query using simple-dns.
+        let qname = simple_dns::Name::new(&ptr_name).map_err(|_| Errno::EINVAL)?;
+        let question = simple_dns::Question::new(
+            qname.clone(),
+            simple_dns::QTYPE::try_from(12).expect("QTYPE::PTR"),
+            simple_dns::QCLASS::try_from(1).expect("QCLASS::IN"),
+            false,
+        );
+        let mut packet = simple_dns::Packet::new_query(0);
+        packet.questions.push(question);
+
+        // Set Recursion Desired (RD) bit (Byte 2, Bit 0).
+        let mut packet_bytes = packet.build_bytes_vec().map_err(|_| Errno::EINVAL)?;
+        if packet_bytes.len() > 2 {
+            packet_bytes[2] |= 1;
         }
-    }
 
-    // SAFETY: On success, `host_buf` contains a valid null-terminated string.
-    let cstr = unsafe { CStr::from_ptr(host_buf.as_ptr().cast()) };
-    let name = cstr.to_string_lossy().into_owned();
-    Ok(name)
+        let socket = UdpSocket::bind("0.0.0.0:0").map_err(io_to_errno)?;
+        socket
+            .set_read_timeout(Some(Duration::from_secs(2)))
+            .map_err(io_to_errno)?;
+        socket.connect(ns).map_err(io_to_errno)?;
+        socket.send(&packet_bytes).map_err(io_to_errno)?;
+
+        let mut recv_buf = [0u8; 1024];
+        let amt = socket.recv(&mut recv_buf).map_err(io_to_errno)?;
+
+        let response = simple_dns::Packet::parse(&recv_buf[..amt]).map_err(|_| Errno::EIO)?;
+
+        if response.answers.is_empty() {
+            return Err(Errno::ENOENT);
+        }
+
+        // Extract PTR record.
+        for answer in response.answers {
+            match answer.rdata {
+                simple_dns::rdata::RData::PTR(ptr) => return Ok(ptr.0.to_string()),
+                _ => continue,
+            }
+        }
+
+        Err(Errno::ENOENT)
+    }
 }
 
 /// Defines hash functions supported by Syd.
@@ -661,6 +772,7 @@ struct InspectOpts {
 struct AccessMap {
     magic: PandoraMap<String, PandoraSet<String>>,
     force: PandoraSet<String>,
+    fs_types: PandoraSet<String>,
     ioctl: PandoraSet<u64>,
     ioctl_names: StringPatriciaSet,
 }
@@ -709,14 +821,9 @@ impl AccessMap {
 
         match json {
             Access::Path {
-                ctx,
-                cap,
-                ctl,
-                path,
-                args,
-                ..
+                ctx, cap, fs, path, ..
             } if ctx == "access" => {
-                let capabilities = match cap {
+                let mut capabilities = match cap {
                     None => return None,
                     Some(Capability::One(cap)) => {
                         let mut caps = PandoraSet::default();
@@ -730,11 +837,11 @@ impl AccessMap {
                     self.force.insert(path.clone());
                 }
 
-                if capabilities.contains("ioctl") {
-                    if let Some(reqs) = ctl {
-                        self.ioctl_names.extend(reqs);
-                    } else if let Some(args) = args {
-                        self.ioctl.insert(args[1]);
+                if let Some(fs_type) = fs {
+                    self.fs_types.insert(fs_type);
+                    capabilities.remove("fs");
+                    if capabilities.is_empty() {
+                        return None;
                     }
                 }
 
@@ -770,6 +877,18 @@ impl AccessMap {
                 let unix = process_path(&unix, u8::MAX).to_string();
                 self.magic.entry(unix).or_default().extend(capabilities);
             }
+            Access::Ioctl { ctx, ctl, .. } if ctx == "access" => {
+                for req in ctl {
+                    match req {
+                        IoctlEntry::Val(val) => {
+                            self.ioctl.insert(val);
+                        }
+                        IoctlEntry::Name(name) => {
+                            self.ioctl_names.insert(name);
+                        }
+                    }
+                }
+            }
             Access::InetAddr { ctx, cap, addr, .. } if ctx == "access" => {
                 let capabilities = match cap {
                     None => return None,
@@ -949,8 +1068,8 @@ fn parse_main_opts() -> Result<MainOpts, lexopt::Error> {
 
     match first_arg {
         None => Ok(MainOpts::Help),
-        Some(Short('h')) => Ok(MainOpts::Help),
-        Some(Short('V')) => Ok(MainOpts::Version),
+        Some(Short('h') | Long("help")) => Ok(MainOpts::Help),
+        Some(Short('V') | Long("version")) => Ok(MainOpts::Version),
         Some(ref arg @ Value(ref cmd)) => match cmd.as_bytes() {
             b"profile" => {
                 let prof = parse_profile_opts(parser)?;
@@ -1030,12 +1149,12 @@ fn parse_profile_opts(mut parser: lexopt::Parser) -> Result<ProfileOpts, lexopt:
     while let Some(arg) = parser.next()? {
         match arg {
             // -h => subcommand help
-            Short('h') => {
+            Short('h') | Long("help") => {
                 print_help_profile();
                 std::process::exit(0);
             }
             // -V => version
-            Short('V') => {
+            Short('V') | Long("version") => {
                 print_version();
                 std::process::exit(0);
             }
@@ -1168,11 +1287,11 @@ fn parse_inspect_opts(mut parser: lexopt::Parser) -> Result<InspectOpts, lexopt:
 
     while let Some(arg) = parser.next()? {
         match arg {
-            Short('h') => {
+            Short('h') | Long("help") => {
                 print_help_inspect();
                 std::process::exit(0);
             }
-            Short('V') => {
+            Short('V') | Long("version") => {
                 print_version();
                 std::process::exit(0);
             }
@@ -1309,7 +1428,7 @@ fn do_inspect(
     }
     let config = config.join("\n");
 
-    // Step 1: Print out the magic header.
+    // Print out the magic header.
     wln!(
         &mut output,
         "#
@@ -1335,8 +1454,16 @@ fn do_inspect(
         ));
     }
 
-    // Step 2: Print out all the sandbox rules from `magic`.
     wln!(&mut output, "###\n# Sandbox Rules\n###")?;
+
+    // Print filesystem sandboxing rules.
+    if !access.fs_types.is_empty() {
+        let mut fs_types = access.fs_types.into_iter().collect::<Vec<_>>();
+        fs_types.sort();
+        wln!(&mut output, "allow/fs+{}\n", fs_types.join(","))?;
+    }
+
+    // Print out all the sandbox rules from `magic`.
     let mut list = Vec::from_iter(access.magic);
     // Alphabetical sort.
     list.sort_by_key(|(path, _)| path.to_string());
@@ -1441,12 +1568,12 @@ fn do_inspect(
         style(len.to_string()).bold().yellow(),
     );
 
-    // Step 3: Print out all ioctl requests.
+    // Print out all ioctl requests.
     if !access.ioctl.is_empty() || !access.ioctl_names.is_empty() {
         wln!(&mut output, "\n###\n# Sandbox ioctl(2) Rules\n###")?;
         wln!(&mut output, "sandbox/ioctl:on\n")?;
 
-        // Step 3.1: Print unknown numeric ioctls.
+        // Print unknown numeric ioctls.
         let mut nums = Vec::with_capacity(access.ioctl.len());
         for &n in access.ioctl.iter() {
             nums.push(n);
@@ -1456,7 +1583,7 @@ fn do_inspect(
         let mut line = String::new();
         for chunk in nums.chunks(5) {
             line.clear();
-            line.push_str("ioctl/allow+");
+            line.push_str("allow/ioctl+");
             for (i, n) in chunk.iter().enumerate() {
                 if i > 0 {
                     line.push(',');
@@ -1466,7 +1593,7 @@ fn do_inspect(
             wln!(&mut output, "{line}")?;
         }
 
-        // Step 3.2: Print named ioctls.
+        // Print named ioctls.
         if !access.ioctl_names.is_empty() {
             // Bucket names by inferred prefix without double-storing keys.
             let mut groups: PandoraMap<String, Vec<String>> = PandoraMap::default();
@@ -1502,7 +1629,7 @@ fn do_inspect(
             for (_k, mut v) in grouped {
                 v.sort_unstable();
                 buf.clear();
-                buf.push_str("ioctl/allow+");
+                buf.push_str("allow/ioctl+");
                 for (i, name) in v.iter().enumerate() {
                     if i > 0 {
                         buf.push(',');
@@ -1514,7 +1641,7 @@ fn do_inspect(
         }
     }
 
-    // Step 4: Print Force entries if available,
+    // Print Force entries if available,
     // concurrency-limited parallel checksums + multiple progress bars.
     if !access.force.is_empty() {
         wln!(&mut output, "\n###\n# Executable Verification\n###")?;
@@ -1639,7 +1766,7 @@ fn do_inspect(
 
     let _ = wln!(
         stderr(),
-        "{} To use it, do: {} -P \"{}\" -- command args...",
+        "{} To use it, do: {} -P \"{}\" -- {cmd}",
         style("pandora:").bold().magenta(),
         style("syd").bold().green(),
         style(output_path).bold().yellow(),
