@@ -23,7 +23,8 @@ use nix::{errno::Errno, fcntl::OFlag, unistd::Pid};
 use crate::{
     compat::{openat2, OpenHow, ResolveFlag},
     err::err2no,
-    fs::{readlinkat, AT_BADFD},
+    fd::AT_BADFD,
+    fs::readlinkat,
     io::{read_all, write_all},
     path::{XPath, XPathBuf},
 };
@@ -147,6 +148,7 @@ fn cgroup_root_path() -> XPathBuf {
 impl Cgroup {
     /// Create new cgroup for a process
     pub fn new(name: &[u8]) -> Result<Self, Errno> {
+        // TODO: open cgroup_root first and verify inode!
         let cgroup_path = cgroup_root_path().join(name);
 
         // Create cgroup directory.
