@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/compat.rs: Compatibility code for different libcs
 //
-// Copyright (c) 2023, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2023, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 // waitid and WaitStatus are based in part upon nix which is:
 //     Copyright (c) nix authors
 //     SPDX-License-Identifier: MIT
@@ -2817,27 +2817,37 @@ pub fn dup3(oldfd: RawFd, newfd: RawFd, flags: libc::c_int) -> Result<OwnedFd, E
 }
 
 /// Set an identifier (or reset it) to the address memory range.
+///
+/// No-op in release mode.
 pub fn set_vma_anon_name(
-    addr: NonNull<libc::c_void>,
-    length: NonZeroUsize,
-    name: Option<&CStr>,
+    _addr: NonNull<libc::c_void>,
+    _length: NonZeroUsize,
+    _name: Option<&CStr>,
 ) -> Result<(), Errno> {
-    let nameref = match name {
-        Some(n) => n.as_ptr(),
-        _ => std::ptr::null(),
-    };
+    #[cfg(not(debug_assertions))]
+    {
+        return Ok(());
+    }
 
-    // SAFETY: In libc we trust.
-    let res = unsafe {
-        libc::prctl(
-            libc::PR_SET_VMA,
-            libc::PR_SET_VMA_ANON_NAME,
-            addr.as_ptr(),
-            length,
-            nameref,
-        )
-    };
-    Errno::result(res).map(drop)
+    #[cfg(debug_assertions)]
+    {
+        let nameref = match _name {
+            Some(n) => n.as_ptr(),
+            _ => std::ptr::null(),
+        };
+
+        // SAFETY: In libc we trust.
+        let res = unsafe {
+            libc::prctl(
+                libc::PR_SET_VMA,
+                libc::PR_SET_VMA_ANON_NAME,
+                _addr.as_ptr(),
+                _length,
+                nameref,
+            )
+        };
+        Errno::result(res).map(drop)
+    }
 }
 
 /// An enumeration allowing the definition of the expiration time of an alarm,
