@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/cookie.rs: Syscall argument cookies
 //
-// Copyright (c) 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -11,16 +11,21 @@
 #![allow(dead_code)]
 
 use std::{
+    ffi::CStr,
     mem::MaybeUninit,
+    num::NonZeroUsize,
     os::fd::{AsFd, AsRawFd, FromRawFd, OwnedFd, RawFd},
+    ptr::NonNull,
     sync::LazyLock,
 };
 
+use libc::{c_uint, c_void};
 use libseccomp::ScmpSyscall;
 use nix::{
     errno::Errno,
     fcntl::{AtFlags, OFlag},
     sys::{
+        mman::{mmap_anonymous, MapFlags, ProtFlags},
         socket::{bind, connect, AddressFamily, Shutdown, SockFlag, SockType, SockaddrLike},
         stat::Mode,
     },
@@ -29,8 +34,9 @@ use nix::{
 };
 
 use crate::{
-    compat::{MFdFlags, OpenHow, RenameFlags},
-    rng::randint,
+    compat::{set_vma_anon_name, MFdFlags, OpenHow, RenameFlags},
+    rng::fillrandom,
+    sealbox::{getpagesize, mprotect_none, mprotect_readonly, mseal},
     uts::UtsName,
 };
 
@@ -42,252 +48,229 @@ pub(crate) type Cookie = u32;
 #[cfg(target_pointer_width = "64")]
 pub(crate) type Cookie = u64;
 
-/// Generate a random `Cookie` using the OS's secure RNG.
-///
-/// This uses `syd::fs::fillrandom` under the hood to pull in
-/// exactly the number of bytes needed for `Cookie`,
-/// interprets them in little-endian order, and returns the result.
-pub(crate) fn getcookie() -> Result<Cookie, Errno> {
-    #[cfg(target_pointer_width = "32")]
-    {
-        randint(0u32..=u32::MAX)
+/// Cookie index enumeration for accessing the sealed pool.
+#[repr(usize)]
+#[derive(Debug, Clone, Copy)]
+#[expect(missing_docs)]
+pub enum CookieIdx {
+    CloseRangeArg3 = 0,
+    CloseRangeArg4,
+    CloseRangeArg5,
+    Openat2Arg4,
+    Openat2Arg5,
+    FchdirArg1,
+    FchdirArg2,
+    FchdirArg3,
+    FchdirArg4,
+    FchdirArg5,
+    SocketArg3,
+    SocketArg4,
+    SocketArg5,
+    SocketpairArg4,
+    SocketpairArg5,
+    Accept4Arg4,
+    Accept4Arg5,
+    BindArg3,
+    BindArg4,
+    BindArg5,
+    ConnectArg3,
+    ConnectArg4,
+    ConnectArg5,
+    ShutdownArg2,
+    ShutdownArg3,
+    ShutdownArg4,
+    ShutdownArg5,
+    MemfdCreateArg2,
+    MemfdCreateArg3,
+    MemfdCreateArg4,
+    MemfdCreateArg5,
+    Renameat2Arg5,
+    TruncateArg2,
+    TruncateArg3,
+    TruncateArg4,
+    TruncateArg5,
+    Truncate64Arg3,
+    Truncate64Arg4,
+    Truncate64Arg5,
+    FtruncateArg2,
+    FtruncateArg3,
+    FtruncateArg4,
+    FtruncateArg5,
+    Ftruncate64Arg3,
+    Ftruncate64Arg4,
+    Ftruncate64Arg5,
+    UmaskArg1,
+    UmaskArg2,
+    UmaskArg3,
+    UmaskArg4,
+    UmaskArg5,
+    UnameArg1,
+    UnameArg2,
+    UnameArg3,
+    UnameArg4,
+    UnameArg5,
+    UnlinkatArg3,
+    UnlinkatArg4,
+    UnlinkatArg5,
+    LinkatArg5,
+    Pipe2Arg2,
+    Pipe2Arg3,
+    Pipe2Arg4,
+    Pipe2Arg5,
+    SeccompIoctlNotifAddfdArg3,
+    SeccompIoctlNotifAddfdArg4,
+    SeccompIoctlNotifAddfdArg5,
+    SeccompIoctlNotifSendArg3,
+    SeccompIoctlNotifSendArg4,
+    SeccompIoctlNotifSendArg5,
+    ProcmapQueryArg3,
+    ProcmapQueryArg4,
+    ProcmapQueryArg5,
+}
+
+impl CookieIdx {
+    /// Number of cookie indices.
+    pub const COUNT: usize = Self::ProcmapQueryArg5 as usize + 1;
+}
+
+/// The sealed syscall cookie pool containing all syscall argument cookies in a
+/// single, hardened memory region. This region is:
+/// 1. Allocated with mmap as a single contiguous anonymous mapping
+/// 2. Guarded by PROT_NONE pages from below and above
+/// 3. Populated with a single fillrandom(2) call
+/// 4. Made read-only with mprotect(PROT_READ)
+/// 5. Named "syd" via prctl(PR_SET_VMA)
+/// 6. Sealed with mseal(2) to prevent remapping
+pub struct SyscookiePool {
+    /// Raw pointer into the data region (after the lower guard page).
+    ptr: *const Cookie,
+    /// Full mapping pointer (including guards) for bookkeeping.
+    #[expect(dead_code)]
+    map_ptr: NonNull<c_void>,
+    /// Total mapping length (guards + data region).
+    #[expect(dead_code)]
+    map_len: NonZeroUsize,
+}
+
+// SAFETY: SyscookiePool is read-only after initialization.
+unsafe impl Sync for SyscookiePool {}
+// SAFETY: ditto.
+unsafe impl Send for SyscookiePool {}
+
+/// VMA name for the cookie pool.
+const VMA_NAME: &CStr = c" Syd: cookie/pool";
+
+impl SyscookiePool {
+    // Allocate and initialize the cookie pool.
+    fn new() -> Result<Self, Errno> {
+        let page = getpagesize()?;
+        let cookie_size = size_of::<Cookie>();
+        let data_size = cookie_size
+            .checked_mul(CookieIdx::COUNT)
+            .ok_or(Errno::EINVAL)?;
+        let data_pages = data_size
+            .checked_next_multiple_of(page)
+            .ok_or(Errno::EINVAL)?;
+
+        // Total layout: [guard page]+[data pages]+[guard page]
+        let total_size = page
+            .checked_add(data_pages)
+            .and_then(|s| s.checked_add(page))
+            .ok_or(Errno::EINVAL)?;
+        let map_len = NonZeroUsize::new(total_size).ok_or(Errno::EINVAL)?;
+
+        // Allocate the entire region as PROT_READ | PROT_WRITE initially.
+        //
+        // SAFETY: Valid length and flags guaranteed.
+        let map_ptr = unsafe {
+            mmap_anonymous(
+                None,
+                map_len,
+                ProtFlags::PROT_READ | ProtFlags::PROT_WRITE,
+                MapFlags::MAP_PRIVATE,
+            )?
+        };
+
+        // Calculate the data region pointer (after lower guard).
+        //
+        // SAFETY: We allocated at least page + data_pages + page bytes.
+        let data_ptr = unsafe { map_ptr.as_ptr().add(page) };
+
+        // Fill the data region with random bytes using a single getrandom(2) call.
+        //
+        // SAFETY: data_ptr is valid for data_pages bytes.
+        let data_slice =
+            unsafe { std::slice::from_raw_parts_mut(data_ptr.cast::<u8>(), data_pages) };
+        fillrandom(data_slice)?;
+
+        // Make the lower guard page PROT_NONE.
+        let guard_len = NonZeroUsize::new(page).ok_or(Errno::EINVAL)?;
+        mprotect_none(map_ptr, guard_len)?;
+
+        // Make the upper guard page PROT_NONE.
+        //
+        // SAFETY: Valid pointer arithmetic.
+        let upper_guard_ptr = unsafe {
+            NonNull::new_unchecked(map_ptr.as_ptr().add(page).add(data_pages).cast::<c_void>())
+        };
+        mprotect_none(upper_guard_ptr, guard_len)?;
+
+        // Make data region read-only.
+        let data_region = NonZeroUsize::new(data_pages).ok_or(Errno::EINVAL)?;
+        // SAFETY: Valid pointer.
+        let data_region_ptr =
+            unsafe { NonNull::new_unchecked(map_ptr.as_ptr().add(page).cast::<c_void>()) };
+        mprotect_readonly(data_region_ptr, data_region)?;
+
+        // Set VMA name to "syd::cookie::pool" in debug mode.
+        let _ = set_vma_anon_name(data_region_ptr, data_region, Some(VMA_NAME));
+
+        // Seal the entire mapping to prevent remapping.
+        //
+        // ENOSYS: mseal(2) not implemented (Linux >= 6.10)
+        // EPERM: Sealing only supported on 64-bit CPUs.
+        match mseal(map_ptr, map_len) {
+            Ok(_) | Err(Errno::EPERM | Errno::ENOSYS) => {}
+            Err(errno) => return Err(errno),
+        }
+
+        Ok(SyscookiePool {
+            ptr: data_ptr.cast::<Cookie>(),
+            map_ptr,
+            map_len,
+        })
     }
 
-    #[cfg(target_pointer_width = "64")]
-    {
-        randint(0u64..=u64::MAX)
+    /// Get a cookie by index.
+    #[inline(always)]
+    pub fn get(&self, idx: CookieIdx) -> Cookie {
+        // SAFETY: Index is bounds-checked by enum, data is read-only.
+        unsafe { *self.ptr.add(idx as usize) }
     }
 }
 
-// These cookies are confined by seccomp for use with openat2(2).
+/// The global sealed syscall cookie pool.
 #[expect(clippy::disallowed_methods)]
-pub(crate) static OPENAT2_COOKIE_ARG4: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static OPENAT2_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static FCHDIR_COOKIE_ARG1: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static FCHDIR_COOKIE_ARG2: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static FCHDIR_COOKIE_ARG3: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static FCHDIR_COOKIE_ARG4: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static FCHDIR_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: socket(2) $arg3
-#[expect(clippy::disallowed_methods)]
-pub static SOCKET_COOKIE_ARG3: LazyLock<Cookie> = LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: socket(2) $arg4
-#[expect(clippy::disallowed_methods)]
-pub static SOCKET_COOKIE_ARG4: LazyLock<Cookie> = LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: socket(2) $arg5
-#[expect(clippy::disallowed_methods)]
-pub static SOCKET_COOKIE_ARG5: LazyLock<Cookie> = LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: socket(2) $arg4
-#[expect(clippy::disallowed_methods)]
-pub static SOCKETPAIR_COOKIE_ARG4: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: socket(2) $arg5
-#[expect(clippy::disallowed_methods)]
-pub static SOCKETPAIR_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: accept4(2) $arg4
-#[expect(clippy::disallowed_methods)]
-pub static ACCEPT4_COOKIE_ARG4: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: accept4(2) $arg5
-#[expect(clippy::disallowed_methods)]
-pub static ACCEPT4_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: bind(2) $arg3
-#[expect(clippy::disallowed_methods)]
-pub static BIND_COOKIE_ARG3: LazyLock<Cookie> = LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: bind(2) $arg4
-#[expect(clippy::disallowed_methods)]
-pub static BIND_COOKIE_ARG4: LazyLock<Cookie> = LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: bind(2) $arg5
-#[expect(clippy::disallowed_methods)]
-pub static BIND_COOKIE_ARG5: LazyLock<Cookie> = LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: connect(2) $arg3
-#[expect(clippy::disallowed_methods)]
-pub static CONNECT_COOKIE_ARG3: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: connect(2) $arg4
-#[expect(clippy::disallowed_methods)]
-pub static CONNECT_COOKIE_ARG4: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: connect(2) $arg5
-#[expect(clippy::disallowed_methods)]
-pub static CONNECT_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: shutdown(2) $arg2
-#[expect(clippy::disallowed_methods)]
-pub static SHUTDOWN_COOKIE_ARG2: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: shutdown(2) $arg3
-#[expect(clippy::disallowed_methods)]
-pub static SHUTDOWN_COOKIE_ARG3: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: shutdown(2) $arg4
-#[expect(clippy::disallowed_methods)]
-pub static SHUTDOWN_COOKIE_ARG4: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: shutdown(2) $arg5
-#[expect(clippy::disallowed_methods)]
-pub static SHUTDOWN_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
+pub static SYSCOOKIE_POOL: LazyLock<SyscookiePool> =
+    LazyLock::new(|| SyscookiePool::new().expect("failed to initialize syscall cookie pool"));
 
-#[expect(clippy::disallowed_methods)]
-pub(crate) static MEMFD_CREATE_COOKIE_ARG2: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static MEMFD_CREATE_COOKIE_ARG3: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static MEMFD_CREATE_COOKIE_ARG4: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static MEMFD_CREATE_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static RENAMEAT2_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static TRUNCATE_COOKIE_ARG2: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static TRUNCATE_COOKIE_ARG3: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static TRUNCATE_COOKIE_ARG4: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static TRUNCATE_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static TRUNCATE64_COOKIE_ARG3: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static TRUNCATE64_COOKIE_ARG4: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static TRUNCATE64_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static FTRUNCATE_COOKIE_ARG2: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static FTRUNCATE_COOKIE_ARG3: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static FTRUNCATE_COOKIE_ARG4: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static FTRUNCATE_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static FTRUNCATE64_COOKIE_ARG3: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static FTRUNCATE64_COOKIE_ARG4: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static FTRUNCATE64_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static UMASK_COOKIE_ARG1: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static UMASK_COOKIE_ARG2: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static UMASK_COOKIE_ARG3: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static UMASK_COOKIE_ARG4: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static UMASK_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static UNAME_COOKIE_ARG1: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static UNAME_COOKIE_ARG2: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static UNAME_COOKIE_ARG3: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static UNAME_COOKIE_ARG4: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static UNAME_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static UNLINKAT_COOKIE_ARG3: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static UNLINKAT_COOKIE_ARG4: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static UNLINKAT_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static LINKAT_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: pipe2(2) $arg2
-#[expect(clippy::disallowed_methods)]
-pub static PIPE2_COOKIE_ARG2: LazyLock<Cookie> = LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: pipe2(2) $arg3
-#[expect(clippy::disallowed_methods)]
-pub static PIPE2_COOKIE_ARG3: LazyLock<Cookie> = LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: pipe2(2) $arg4
-#[expect(clippy::disallowed_methods)]
-pub static PIPE2_COOKIE_ARG4: LazyLock<Cookie> = LazyLock::new(|| getcookie().expect("getcookie"));
-/// System call argument cookie: pipe2(2) $arg5
-#[expect(clippy::disallowed_methods)]
-pub static PIPE2_COOKIE_ARG5: LazyLock<Cookie> = LazyLock::new(|| getcookie().expect("getcookie"));
-
-/// These are used in `syd::fs::seccomp_notify_addfd`.
-#[expect(clippy::disallowed_methods)]
-pub(crate) static SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG3: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG4: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-
-/// These are used in `syd::fs::seccomp_notify_respond`.
-#[expect(clippy::disallowed_methods)]
-pub(crate) static SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG3: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG4: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-
-/// These are used in `syd::proc::procmap_query`.
-#[expect(clippy::disallowed_methods)]
-pub(crate) static PROCMAP_QUERY_COOKIE_ARG3: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static PROCMAP_QUERY_COOKIE_ARG4: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
-#[expect(clippy::disallowed_methods)]
-pub(crate) static PROCMAP_QUERY_COOKIE_ARG5: LazyLock<Cookie> =
-    LazyLock::new(|| getcookie().expect("getcookie"));
+/// Safe close_range(2) confined by syscall cookies.
+#[inline(always)]
+pub(crate) fn safe_close_range(first: c_uint, last: c_uint, flags: c_uint) -> Result<(), Errno> {
+    // SAFETY: In libc we trust.
+    Errno::result(unsafe {
+        libc::syscall(
+            libc::SYS_close_range,
+            first,
+            last,
+            flags,
+            SYSCOOKIE_POOL.get(CookieIdx::CloseRangeArg3),
+            SYSCOOKIE_POOL.get(CookieIdx::CloseRangeArg4),
+            SYSCOOKIE_POOL.get(CookieIdx::CloseRangeArg5),
+        )
+    })
+    .map(drop)
+}
 
 /// Safe openat2(2) confined by syscall cookies.
 #[inline(always)]
@@ -305,8 +288,8 @@ pub(crate) fn safe_openat2<Fd: AsFd, P: NixPath + ?Sized>(
             cstr.as_ptr(),
             std::ptr::addr_of_mut!(how),
             size_of::<OpenHow>(),
-            *OPENAT2_COOKIE_ARG4,
-            *OPENAT2_COOKIE_ARG5,
+            SYSCOOKIE_POOL.get(CookieIdx::Openat2Arg4),
+            SYSCOOKIE_POOL.get(CookieIdx::Openat2Arg5),
         )
     })? as RawFd;
 
@@ -347,9 +330,9 @@ pub fn safe_socket(
                 domain,
                 stype,
                 proto,
-                *SOCKET_COOKIE_ARG3,
-                *SOCKET_COOKIE_ARG4,
-                *SOCKET_COOKIE_ARG5,
+                SYSCOOKIE_POOL.get(CookieIdx::SocketArg3),
+                SYSCOOKIE_POOL.get(CookieIdx::SocketArg4),
+                SYSCOOKIE_POOL.get(CookieIdx::SocketArg5),
             )
         })
         .map(|fd| fd as RawFd)
@@ -401,8 +384,8 @@ pub fn safe_socketpair(
                 ty,
                 proto,
                 fds.as_mut_ptr(),
-                *SOCKETPAIR_COOKIE_ARG4,
-                *SOCKETPAIR_COOKIE_ARG5,
+                SYSCOOKIE_POOL.get(CookieIdx::SocketpairArg4),
+                SYSCOOKIE_POOL.get(CookieIdx::SocketpairArg5),
             )
         })?;
     } else {
@@ -453,8 +436,8 @@ pub unsafe fn safe_accept4<Fd: AsFd>(
                 addr,
                 len,
                 flags.bits(),
-                *ACCEPT4_COOKIE_ARG4,
-                *ACCEPT4_COOKIE_ARG5,
+                SYSCOOKIE_POOL.get(CookieIdx::Accept4Arg4),
+                SYSCOOKIE_POOL.get(CookieIdx::Accept4Arg5),
             )
         })
         .map(|fd| {
@@ -505,9 +488,9 @@ pub fn safe_bind<Fd: AsFd>(fd: Fd, addr: &dyn SockaddrLike) -> Result<(), Errno>
                 fd.as_fd().as_raw_fd(),
                 addr.as_ptr(),
                 addr.len(),
-                *BIND_COOKIE_ARG3,
-                *BIND_COOKIE_ARG4,
-                *BIND_COOKIE_ARG5,
+                SYSCOOKIE_POOL.get(CookieIdx::BindArg3),
+                SYSCOOKIE_POOL.get(CookieIdx::BindArg4),
+                SYSCOOKIE_POOL.get(CookieIdx::BindArg5),
             )
         })
         .map(drop)
@@ -541,9 +524,9 @@ pub fn safe_connect<Fd: AsFd>(fd: Fd, addr: &dyn SockaddrLike) -> Result<(), Err
                 fd.as_fd().as_raw_fd(),
                 addr.as_ptr(),
                 addr.len(),
-                *CONNECT_COOKIE_ARG3,
-                *CONNECT_COOKIE_ARG4,
-                *CONNECT_COOKIE_ARG5,
+                SYSCOOKIE_POOL.get(CookieIdx::ConnectArg3),
+                SYSCOOKIE_POOL.get(CookieIdx::ConnectArg4),
+                SYSCOOKIE_POOL.get(CookieIdx::ConnectArg5),
             )
         })
         .map(drop)
@@ -584,10 +567,10 @@ pub fn safe_shutdown<Fd: AsFd>(fd: Fd, how: Shutdown) -> Result<(), Errno> {
                 sys_shutdown,
                 fd.as_fd().as_raw_fd(),
                 how,
-                *SHUTDOWN_COOKIE_ARG2,
-                *SHUTDOWN_COOKIE_ARG3,
-                *SHUTDOWN_COOKIE_ARG4,
-                *SHUTDOWN_COOKIE_ARG5,
+                SYSCOOKIE_POOL.get(CookieIdx::ShutdownArg2),
+                SYSCOOKIE_POOL.get(CookieIdx::ShutdownArg3),
+                SYSCOOKIE_POOL.get(CookieIdx::ShutdownArg4),
+                SYSCOOKIE_POOL.get(CookieIdx::ShutdownArg5),
             )
         })
         .map(drop)
@@ -612,10 +595,10 @@ pub fn safe_memfd_create<P: NixPath + ?Sized>(name: &P, flags: MFdFlags) -> Resu
                 libc::SYS_memfd_create,
                 cstr.as_ptr(),
                 flags.bits(),
-                *MEMFD_CREATE_COOKIE_ARG2,
-                *MEMFD_CREATE_COOKIE_ARG3,
-                *MEMFD_CREATE_COOKIE_ARG4,
-                *MEMFD_CREATE_COOKIE_ARG5,
+                SYSCOOKIE_POOL.get(CookieIdx::MemfdCreateArg2),
+                SYSCOOKIE_POOL.get(CookieIdx::MemfdCreateArg3),
+                SYSCOOKIE_POOL.get(CookieIdx::MemfdCreateArg4),
+                SYSCOOKIE_POOL.get(CookieIdx::MemfdCreateArg5),
             )
         }
     })?;
@@ -648,7 +631,7 @@ pub(crate) fn safe_renameat2<Fd1: AsFd, Fd2: AsFd, P1: NixPath + ?Sized, P2: Nix
                 new_dirfd.as_fd().as_raw_fd(),
                 new_cstr.as_ptr(),
                 flags.bits(),
-                *RENAMEAT2_COOKIE_ARG5,
+                SYSCOOKIE_POOL.get(CookieIdx::Renameat2Arg5),
             )
         })
     })??;
@@ -724,10 +707,10 @@ pub(crate) fn safe_truncate<P: NixPath + ?Sized>(path: &P, len: libc::off_t) ->
             sys_truncate,
             cstr.as_ptr(),
             len,
-            *TRUNCATE_COOKIE_ARG2,
-            *TRUNCATE_COOKIE_ARG3,
-            *TRUNCATE_COOKIE_ARG4,
-            *TRUNCATE_COOKIE_ARG5,
+            SYSCOOKIE_POOL.get(CookieIdx::TruncateArg2),
+            SYSCOOKIE_POOL.get(CookieIdx::TruncateArg3),
+            SYSCOOKIE_POOL.get(CookieIdx::TruncateArg4),
+            SYSCOOKIE_POOL.get(CookieIdx::TruncateArg5),
         )
     })?;
 
@@ -777,9 +760,9 @@ pub(crate) fn safe_truncate64<P: NixPath + ?Sized>(
                 cstr.as_ptr(),
                 low,
                 high,
-                *TRUNCATE64_COOKIE_ARG3,
-                *TRUNCATE64_COOKIE_ARG4,
-                *TRUNCATE64_COOKIE_ARG5,
+                SYSCOOKIE_POOL.get(CookieIdx::Truncate64Arg3),
+                SYSCOOKIE_POOL.get(CookieIdx::Truncate64Arg4),
+                SYSCOOKIE_POOL.get(CookieIdx::Truncate64Arg5),
             )
         })?)
         .map(drop)
@@ -807,8 +790,8 @@ pub(crate) fn safe_truncate64<P: NixPath + ?Sized>(
                 0 as libc::c_long,
                 low,
                 high,
-                *TRUNCATE64_COOKIE_ARG4,
-                *TRUNCATE64_COOKIE_ARG5,
+                SYSCOOKIE_POOL.get(CookieIdx::Truncate64Arg4),
+                SYSCOOKIE_POOL.get(CookieIdx::Truncate64Arg5),
             )
         })?)
         .map(drop)
@@ -830,10 +813,10 @@ pub(crate) fn safe_ftruncate<Fd: AsFd>(fd: Fd, len: libc::off_t) -> Result<(), E
             sys_ftruncate,
             fd.as_fd().as_raw_fd(),
             len,
-            *FTRUNCATE_COOKIE_ARG2,
-            *FTRUNCATE_COOKIE_ARG3,
-            *FTRUNCATE_COOKIE_ARG4,
-            *FTRUNCATE_COOKIE_ARG5,
+            SYSCOOKIE_POOL.get(CookieIdx::FtruncateArg2),
+            SYSCOOKIE_POOL.get(CookieIdx::FtruncateArg3),
+            SYSCOOKIE_POOL.get(CookieIdx::FtruncateArg4),
+            SYSCOOKIE_POOL.get(CookieIdx::FtruncateArg5),
         )
     })
     .map(drop)
@@ -879,9 +862,9 @@ pub(crate) fn safe_ftruncate64<Fd: AsFd>(fd: Fd, len: libc::off64_t) -> Result<(
                 fd.as_fd().as_raw_fd(),
                 low,
                 high,
-                *FTRUNCATE64_COOKIE_ARG3,
-                *FTRUNCATE64_COOKIE_ARG4,
-                *FTRUNCATE64_COOKIE_ARG5,
+                SYSCOOKIE_POOL.get(CookieIdx::Ftruncate64Arg3),
+                SYSCOOKIE_POOL.get(CookieIdx::Ftruncate64Arg4),
+                SYSCOOKIE_POOL.get(CookieIdx::Ftruncate64Arg5),
             )
         })
         .map(drop)
@@ -909,8 +892,8 @@ pub(crate) fn safe_ftruncate64<Fd: AsFd>(fd: Fd, len: libc::off64_t) -> Result<(
                 0 as libc::c_long,
                 low,
                 high,
-                *FTRUNCATE64_COOKIE_ARG4,
-                *FTRUNCATE64_COOKIE_ARG5,
+                SYSCOOKIE_POOL.get(CookieIdx::Ftruncate64Arg4),
+                SYSCOOKIE_POOL.get(CookieIdx::Ftruncate64Arg5),
             )
         })
         .map(drop)
@@ -936,9 +919,9 @@ pub(crate) fn safe_unlinkat<Fd: AsFd, P: NixPath + ?Sized>(
             dirfd.as_fd().as_raw_fd(),
             cstr.as_ptr(),
             atflag.bits(),
-            *UNLINKAT_COOKIE_ARG3,
-            *UNLINKAT_COOKIE_ARG4,
-            *UNLINKAT_COOKIE_ARG5,
+            SYSCOOKIE_POOL.get(CookieIdx::UnlinkatArg3),
+            SYSCOOKIE_POOL.get(CookieIdx::UnlinkatArg4),
+            SYSCOOKIE_POOL.get(CookieIdx::UnlinkatArg5),
         )
     })?;
 
@@ -964,7 +947,7 @@ pub(crate) fn safe_linkat<Fd1: AsFd, Fd2: AsFd, P1: NixPath + ?Sized, P2: NixPat
                 newdirfd.as_fd().as_raw_fd(),
                 newcstr.as_ptr(),
                 flag.bits(),
-                *LINKAT_COOKIE_ARG5,
+                SYSCOOKIE_POOL.get(CookieIdx::LinkatArg5),
             )
         })
     })??;
@@ -980,11 +963,11 @@ pub fn safe_fchdir<Fd: AsFd>(dirfd: Fd) -> Result<(), Errno> {
         libc::syscall(
             libc::SYS_fchdir,
             dirfd.as_fd().as_raw_fd(),
-            *FCHDIR_COOKIE_ARG1,
-            *FCHDIR_COOKIE_ARG2,
-            *FCHDIR_COOKIE_ARG3,
-            *FCHDIR_COOKIE_ARG4,
-            *FCHDIR_COOKIE_ARG5,
+            SYSCOOKIE_POOL.get(CookieIdx::FchdirArg1),
+            SYSCOOKIE_POOL.get(CookieIdx::FchdirArg2),
+            SYSCOOKIE_POOL.get(CookieIdx::FchdirArg3),
+            SYSCOOKIE_POOL.get(CookieIdx::FchdirArg4),
+            SYSCOOKIE_POOL.get(CookieIdx::FchdirArg5),
         )
     })
     .map(drop)
@@ -1000,11 +983,11 @@ pub fn safe_uname() -> Result<UtsName, Errno> {
         libc::syscall(
             libc::SYS_uname,
             &raw mut name,
-            *UNAME_COOKIE_ARG1,
-            *UNAME_COOKIE_ARG2,
-            *UNAME_COOKIE_ARG3,
-            *UNAME_COOKIE_ARG4,
-            *UNAME_COOKIE_ARG5,
+            SYSCOOKIE_POOL.get(CookieIdx::UnameArg1),
+            SYSCOOKIE_POOL.get(CookieIdx::UnameArg2),
+            SYSCOOKIE_POOL.get(CookieIdx::UnameArg3),
+            SYSCOOKIE_POOL.get(CookieIdx::UnameArg4),
+            SYSCOOKIE_POOL.get(CookieIdx::UnameArg5),
         )
     })?;
 
@@ -1021,11 +1004,11 @@ pub fn safe_umask(mode: Mode) -> Mode {
         libc::syscall(
             libc::SYS_umask,
             mode.bits(),
-            *UMASK_COOKIE_ARG1,
-            *UMASK_COOKIE_ARG2,
-            *UMASK_COOKIE_ARG3,
-            *UMASK_COOKIE_ARG4,
-            *UMASK_COOKIE_ARG5,
+            SYSCOOKIE_POOL.get(CookieIdx::UmaskArg1),
+            SYSCOOKIE_POOL.get(CookieIdx::UmaskArg2),
+            SYSCOOKIE_POOL.get(CookieIdx::UmaskArg3),
+            SYSCOOKIE_POOL.get(CookieIdx::UmaskArg4),
+            SYSCOOKIE_POOL.get(CookieIdx::UmaskArg5),
         )
     } as libc::mode_t;
     #[expect(clippy::disallowed_methods)]
@@ -1043,10 +1026,10 @@ pub fn safe_pipe2(flags: OFlag) -> Result<(OwnedFd, OwnedFd), Errno> {
             libc::SYS_pipe2,
             fds.as_mut_ptr(),
             flags.bits(),
-            *PIPE2_COOKIE_ARG2,
-            *PIPE2_COOKIE_ARG3,
-            *PIPE2_COOKIE_ARG4,
-            *PIPE2_COOKIE_ARG5,
+            SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg2),
+            SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg3),
+            SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg4),
+            SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg5),
         )
     })?;
 
