@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/fs.rs: Filesystem utilities
 //
-// Copyright (c) 2023, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2023, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 // Based in part upon uutils coreutils package's src/lib/features/fs.rs which is:
 //   (c) Joseph Crail <jbcrail@gmail.com>
 //   (c) Jian Zeng <anonymousknight96 AT gmail.com>
@@ -54,11 +54,7 @@ use crate::{
     },
     config::*,
     confine::CLONE_NEWTIME,
-    cookie::{
-        safe_socket, SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG3, SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG4,
-        SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG5, SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG3,
-        SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG4, SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG5,
-    },
+    cookie::{safe_socket, CookieIdx, SYSCOOKIE_POOL},
     err::err2no,
     fd::{is_dev_ptmx, AT_BADFD},
     hash::SydHashSet,
@@ -460,9 +456,9 @@ pub(crate) fn seccomp_notify_respond(
                 fd,
                 SECCOMP_IOCTL_NOTIF_SEND as c_ulong,
                 response,
-                *SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG3,
-                *SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG4,
-                *SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG5,
+                SYSCOOKIE_POOL.get(CookieIdx::SeccompIoctlNotifSendArg3),
+                SYSCOOKIE_POOL.get(CookieIdx::SeccompIoctlNotifSendArg4),
+                SYSCOOKIE_POOL.get(CookieIdx::SeccompIoctlNotifSendArg5),
             )
         })
     })
@@ -494,9 +490,9 @@ pub(crate) fn seccomp_notify_addfd(
                 fd,
                 SECCOMP_IOCTL_NOTIF_ADDFD as c_ulong,
                 addfd,
-                *SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG3,
-                *SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG4,
-                *SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG5,
+                SYSCOOKIE_POOL.get(CookieIdx::SeccompIoctlNotifAddfdArg3),
+                SYSCOOKIE_POOL.get(CookieIdx::SeccompIoctlNotifAddfdArg4),
+                SYSCOOKIE_POOL.get(CookieIdx::SeccompIoctlNotifAddfdArg5),
             )
         })
     })
@@ -1038,7 +1034,7 @@ pub fn readlinkat<Fd: AsFd, P: NixPath + ?Sized>(fd: Fd, base: &P) -> Result<XPa
         .map(|n| n as usize)
     })??;
 
-    if n >= target.len() {
+    if n > target.len() {
         // Truncation occurred!
         return Err(Errno::ENAMETOOLONG);
     }
