@@ -27,11 +27,12 @@ use zeroize::Zeroizing;
 use crate::{
     compat::{
         cmsg_len_32, cmsg_space_32, cmsghdr, cmsghdr32, iovec32, mmsghdr, mmsghdr32, mmsghdr_union,
-        msghdr, msghdr32, MsgFlags, UIO_MAXIOV, UIO_MAXLEN,
+        msghdr, msghdr32, MsgFlags, UIO_MAXIOV,
     },
-    config::MMAP_MIN_ADDR,
-    confine::scmp_arch_bits,
-    fs::{get_nonblock, has_recv_timeout, peer_inode},
+    config::MAX_RW_COUNT,
+    confine::{is_valid_ptr, scmp_arch_bits},
+    fd::{get_nonblock, has_recv_timeout},
+    fs::peer_inode,
     kernel::net::to_msgflags,
     path::XPath,
     req::UNotifyEventRequest,
@@ -88,10 +89,10 @@ pub(crate) fn handle_recvmsg(
     let user_iov_base = process_msghdr_iov(request, &mut hdr, &mut msg_bufs, &mut msg_iovs)?;
 
     // Handle msg_name.
-    let (user_nam_base, user_nam_size) = process_msghdr_name(&mut hdr, &mut nam_buf)?;
+    let (user_nam_base, user_nam_size) = process_msghdr_name(request, &mut hdr, &mut nam_buf)?;
 
     // Handle msg_control.
-    let (user_ctl_base, user_ctl_size) = process_msghdr_ctl(is32, &mut hdr, &mut ctl_buf)?;
+    let (user_ctl_base, user_ctl_size) = process_msghdr_ctl(request, &mut hdr, &mut ctl_buf)?;
 
     // Track blocking call for invalidation semantics.
     let is_blocking = !call_flags.contains(MsgFlags::MSG_DONTWAIT) && !get_nonblock(&fd)?;
@@ -409,7 +410,7 @@ fn handle_recvmmsg_internal(
     //
     // 1. Validate message count.
     let msg_count = usize::try_from(args[2]).or(Err(Errno::EINVAL))?;
-    if msg_count > UIO_MAXLEN {
+    if msg_count > *MAX_RW_COUNT {
         // Invalid message count: Too large.
         return Err(Errno::EMSGSIZE);
     }
@@ -470,10 +471,10 @@ fn handle_recvmmsg_internal(
         )?;
 
         // Handle hdr.msg_name.
-        process_mmsghdr_name(&mut hdr, &mut nam_bufs, &mut user_nam_bases)?;
+        process_mmsghdr_name(request, &mut hdr, &mut nam_bufs, &mut user_nam_bases)?;
 
         // Handle hdr.msg_control.
-        process_mmsghdr_ctl(is32, &mut hdr, &mut ctl_bufs, &mut user_ctl_bases)?;
+        process_mmsghdr_ctl(request, &mut hdr, &mut ctl_bufs, &mut user_ctl_bases)?;
 
         msgs.push(hdr);
     }
@@ -796,7 +797,7 @@ fn process_mmsghdr_iov(
         // Invalid iovec buffer.
         return Err(Errno::EFAULT);
     }
-    if iov_len > 0 && (msg_iov as u64) < *MMAP_MIN_ADDR {
+    if iov_len > 0 && !is_valid_ptr(msg_iov as u64, request.scmpreq.data.arch) {
         // Invalid iovec buffer.
         return Err(Errno::EFAULT);
     }
@@ -830,13 +831,13 @@ fn process_mmsghdr_iov(
             if iov.iov_base.is_null() && iov.iov_len > 0 {
                 return Err(Errno::EFAULT);
             }
-            if !iov.iov_base.is_null() && (iov.iov_base as u64) < *MMAP_MIN_ADDR {
+            if !iov.iov_base.is_null() && !is_valid_ptr(iov.iov_base as u64, req.data.arch) {
                 return Err(Errno::EFAULT);
             }
 
             if !iov.iov_base.is_null() && iov.iov_len > 0 {
                 // SAFETY: Cap untrusted `iov_len`.
-                let len = iov.iov_len.min(UIO_MAXLEN);
+                let len = iov.iov_len.min(*MAX_RW_COUNT);
                 let mut buf = Vec::new();
                 buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
                 buf.resize(len, 0);
@@ -882,6 +883,7 @@ fn process_mmsghdr_iov(
 }
 
 fn process_mmsghdr_name(
+    request: &UNotifyEventRequest,
     hdr: &mut libc::mmsghdr,
     nam_bufs: &mut Vec<Option<Vec<u8>>>,
     user_nam_bases: &mut Vec<Option<(u64, usize)>>,
@@ -889,7 +891,9 @@ fn process_mmsghdr_name(
     if hdr.msg_hdr.msg_name.is_null() && hdr.msg_hdr.msg_namelen > 0 {
         return Err(Errno::EFAULT);
     }
-    if !hdr.msg_hdr.msg_name.is_null() && (hdr.msg_hdr.msg_name as u64) < *MMAP_MIN_ADDR {
+    if !hdr.msg_hdr.msg_name.is_null()
+        && !is_valid_ptr(hdr.msg_hdr.msg_name as u64, request.scmpreq.data.arch)
+    {
         return Err(Errno::EFAULT);
     }
     #[expect(clippy::cast_possible_truncation)]
@@ -915,15 +919,19 @@ fn process_mmsghdr_name(
 }
 
 fn process_mmsghdr_ctl(
-    is32: bool,
+    request: &UNotifyEventRequest,
     hdr: &mut libc::mmsghdr,
     ctl_bufs: &mut Vec<Option<Vec<u8>>>,
     user_ctl_bases: &mut Vec<Option<(u64, usize)>>,
 ) -> Result<(), Errno> {
+    let req = request.scmpreq;
+    let is32 = scmp_arch_bits(req.data.arch) == 32;
     if hdr.msg_hdr.msg_control.is_null() && hdr.msg_hdr.msg_controllen > 0 {
         return Err(Errno::EFAULT);
     }
-    if !hdr.msg_hdr.msg_control.is_null() && (hdr.msg_hdr.msg_control as u64) < *MMAP_MIN_ADDR {
+    if !hdr.msg_hdr.msg_control.is_null()
+        && !is_valid_ptr(hdr.msg_hdr.msg_control as u64, req.data.arch)
+    {
         return Err(Errno::EFAULT);
     }
     #[expect(clippy::cast_possible_truncation)]
@@ -932,8 +940,8 @@ fn process_mmsghdr_ctl(
     #[expect(clippy::useless_conversion)]
     if !hdr.msg_hdr.msg_control.is_null() && hdr.msg_hdr.msg_controllen > 0 {
         let mut ctl = Vec::new();
-        // SAFETY: Cap length at UIO_MAXLEN.
-        let mut len = (hdr.msg_hdr.msg_controllen as usize).min(UIO_MAXLEN);
+        // SAFETY: Cap length at MAX_RW_COUNT.
+        let mut len = (hdr.msg_hdr.msg_controllen as usize).min(*MAX_RW_COUNT);
         if is32 {
             // SAFETY: CMSG_SPACE is always safe.
             len = unsafe { libc::CMSG_SPACE(len as u32) } as usize;
@@ -983,7 +991,7 @@ fn process_msghdr_iov(
         // Invalid iovec buffer.
         return Err(Errno::EFAULT);
     }
-    if iov_len > 0 && (msg_iov as u64) < *MMAP_MIN_ADDR {
+    if iov_len > 0 && !is_valid_ptr(msg_iov as u64, req.data.arch) {
         // Invalid iovec buffer.
         return Err(Errno::EFAULT);
     }
@@ -1014,13 +1022,13 @@ fn process_msghdr_iov(
             if iov.iov_base.is_null() && iov.iov_len > 0 {
                 return Err(Errno::EFAULT);
             }
-            if !iov.iov_base.is_null() && (iov.iov_base as u64) < *MMAP_MIN_ADDR {
+            if !iov.iov_base.is_null() && !is_valid_ptr(iov.iov_base as u64, req.data.arch) {
                 return Err(Errno::EFAULT);
             }
 
             if !iov.iov_base.is_null() && iov.iov_len > 0 {
                 // SAFETY: Cap untrusted `iov_len`.
-                let len = iov.iov_len.min(UIO_MAXLEN);
+                let len = iov.iov_len.min(*MAX_RW_COUNT);
                 let mut buf = Vec::new();
                 buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
                 buf.resize(len, 0);
@@ -1055,11 +1063,15 @@ fn process_msghdr_iov(
     Ok(msg_iov as u64)
 }
 
-fn process_msghdr_name(hdr: &mut msghdr, nam_buf: &mut Vec<u8>) -> Result<(u64, usize), Errno> {
+fn process_msghdr_name(
+    request: &UNotifyEventRequest,
+    hdr: &mut msghdr,
+    nam_buf: &mut Vec<u8>,
+) -> Result<(u64, usize), Errno> {
     if hdr.msg_name.is_null() && hdr.msg_namelen > 0 {
         return Err(Errno::EFAULT);
     }
-    if !hdr.msg_name.is_null() && (hdr.msg_name as u64) < *MMAP_MIN_ADDR {
+    if !hdr.msg_name.is_null() && !is_valid_ptr(hdr.msg_name as u64, request.scmpreq.data.arch) {
         return Err(Errno::EFAULT);
     }
     #[expect(clippy::cast_possible_truncation)]
@@ -1084,20 +1096,23 @@ fn process_msghdr_name(hdr: &mut msghdr, nam_buf: &mut Vec<u8>) -> Result<(u64,
 }
 
 fn process_msghdr_ctl(
-    is32: bool,
+    request: &UNotifyEventRequest,
     hdr: &mut msghdr,
     ctl_buf: &mut Vec<u8>,
 ) -> Result<(u64, usize), Errno> {
+    let req = request.scmpreq;
+    let is32 = scmp_arch_bits(req.data.arch) == 32;
+
     if hdr.msg_control.is_null() && hdr.msg_controllen > 0 {
         return Err(Errno::EFAULT);
     }
-    if !hdr.msg_control.is_null() && (hdr.msg_control as u64) < *MMAP_MIN_ADDR {
+    if !hdr.msg_control.is_null() && !is_valid_ptr(hdr.msg_control as u64, req.data.arch) {
         return Err(Errno::EFAULT);
     }
     if !hdr.msg_control.is_null() && hdr.msg_controllen > 0 {
         let mut ctl = Vec::new();
-        // SAFETY: Cap length at UIO_MAXLEN.
-        let mut len = hdr.msg_controllen.min(UIO_MAXLEN);
+        // SAFETY: Cap length at MAX_RW_COUNT.
+        let mut len = hdr.msg_controllen.min(*MAX_RW_COUNT);
         #[expect(clippy::cast_possible_truncation)]
         if is32 {
             // SAFETY: CMSG_SPACE is always safe.
