@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/parsers/syd.rs: syd(2) nom parsers
 //
-// Copyright (c) 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -14,9 +14,10 @@
 use std::{ops::RangeInclusive, str::FromStr};
 
 use btoi::{btoi, btoi_radix};
+use dur::Duration;
 use fixedbitset::FixedBitSet;
 use memchr::arch::all::is_prefix;
-use nix::{errno::Errno, mount::MsFlags};
+use nix::errno::Errno;
 use nom::{
     branch::alt,
     bytes::complete::{tag, tag_no_case, take_while1},
@@ -29,12 +30,13 @@ use nom::{
 };
 
 use crate::{
-    confine::SydMsFlags,
+    confine::SydMountAttrFlags,
     hash::SydHashSet,
     landlock::{AccessFs, AccessNet},
     landlock_policy::LandlockPolicy,
+    mount::api::MountAttrFlags,
     path::XPathBuf,
-    sandbox::{Action, Capability},
+    sandbox::{Action, Capability, SANDBOX_CAPS},
 };
 
 // Valid Netlink families.
@@ -76,9 +78,9 @@ pub(crate) struct BindCmd {
     // Mount destination.
     pub(crate) dst: String,
     // Mount flags.
-    pub(crate) opt: MsFlags,
+    pub(crate) opt: MountAttrFlags,
     // Optional filesystem specific data.
-    pub(crate) dat: Option<XPathBuf>,
+    pub(crate) dat: Option<String>,
 }
 
 // Represents a parsed "force" command.
@@ -149,6 +151,14 @@ impl TryFrom<char> for LandlockOp {
     }
 }
 
+/// Parsed filesystem sandbox rule.
+#[derive(Clone, Debug, PartialEq, Eq)]
+pub(crate) struct FsCmd {
+    pub(crate) action: Action,
+    pub(crate) op: char,
+    pub(crate) fs_type: String,
+}
+
 /// Parsed "allow/lock" command.
 #[derive(Debug, PartialEq, Eq)]
 pub struct LandlockCmd {
@@ -226,8 +236,8 @@ pub(crate) fn parse_bind_cmd(command: &str) -> Result<BindCmd, Errno> {
 
     match inner(command).finish() {
         Ok(("", (op, src, dst, opt))) => {
-            // Build MsFlags and dat from opt.
-            let mut flags = MsFlags::empty();
+            // Build MountAttrFlags and dat from opt.
+            let mut flags = MountAttrFlags::empty();
             let mut dat = Vec::new();
 
             if let Some(opt) = opt {
@@ -251,7 +261,7 @@ pub(crate) fn parse_bind_cmd(command: &str) -> Result<BindCmd, Errno> {
                     }
 
                     // Try to parse as a mount flag.
-                    if let Some(syd_flag) = SydMsFlags::from_name(flag) {
+                    if let Some(syd_flag) = SydMountAttrFlags::from_name(flag) {
                         flags |= syd_flag.0;
                     } else {
                         // Add to auxiliary data.
@@ -266,7 +276,7 @@ pub(crate) fn parse_bind_cmd(command: &str) -> Result<BindCmd, Errno> {
             let dat = if dat.is_empty() {
                 None
             } else {
-                Some(XPathBuf::from(dat))
+                Some(String::from_utf8(dat).or(Err(Errno::EINVAL))?)
             };
 
             // Source and destination will be expanded as necessary,
@@ -526,6 +536,43 @@ pub fn parse_landlock_cmd(input: &str) -> Result<LandlockCmd, Errno> {
     }
 }
 
+// Parse a filesystem rule command string, returning `FsCmd` or `Errno::EINVAL`.
+//
+// Format: `<action>/fs<op><fs_type>`
+// - `<action>`: one of allow, deny, filter, warn, stop, abort, kill, panic, exit
+// - `<op>`: one of '+','-','^'
+// - `<fs_type>`: non-empty string.
+//
+// Returns Err(EINVAL) on any parse or validation failure.
+pub(crate) fn parse_fs_cmd(input: &str) -> Result<FsCmd, Errno> {
+    // Inner parser: action "/" op fs_type.
+    fn inner(input: &str) -> IResult<&str, (&str, char, &str)> {
+        (
+            take_while1(|c| c != '/'),
+            char('/'),
+            tag("fs"),
+            one_of("+-^"),
+            take_while1(|_| true), // fs_type (rest of line, must be non-empty).
+        )
+            .map(|(act, _slash, _caps, op, fs_type)| (act, op, fs_type))
+            .parse(input)
+    }
+
+    match inner(input).finish() {
+        Ok(("", (act, op, fs_type))) => {
+            let action = Action::from_str(act).map_err(|_| Errno::EINVAL)?;
+            let fs_type = fs_type.to_string();
+
+            Ok(FsCmd {
+                action,
+                op,
+                fs_type,
+            })
+        }
+        _ => Err(Errno::EINVAL),
+    }
+}
+
 // Parse a seccomp rule command string, returning `ScmpCmd` or `Errno::EINVAL`.
 //
 // Format: `<action>/<caps><op><pat>`
@@ -542,13 +589,25 @@ pub fn parse_landlock_cmd(input: &str) -> Result<LandlockCmd, Errno> {
 //
 // Returns Err(EINVAL) on any parse or validation failure.
 pub(crate) fn parse_scmp_cmd(input: &str) -> Result<ScmpCmd, Errno> {
+    // Match a capability name via SANDBOX_CAPS iteration order.
+    fn cap_name(input: &str) -> IResult<&str, &str> {
+        for name in SANDBOX_CAPS.keys().rev() {
+            if let Some(rest) = input.strip_prefix(name) {
+                if rest.is_empty() || rest.starts_with([',', '+', '-', '^']) {
+                    return Ok((rest, *name));
+                }
+            }
+        }
+        Err(nom::Err::Error(Error::new(input, ErrorKind::Tag)))
+    }
+
     // Inner parser: action "/" caps op pat.
     #[expect(clippy::type_complexity)]
     fn inner(input: &str) -> IResult<&str, (&str, &str, char, &str)> {
         (
             take_while1(|c| c != '/'),
             char('/'),
-            take_while1(|c| !matches!(c, '+' | '-' | '^')),
+            recognize(separated_list1(char(','), cap_name)),
             one_of("+-^"),
             take_while1(|_| true), // pat (rest of line, must be non-empty).
         )
@@ -573,6 +632,12 @@ pub(crate) fn parse_scmp_cmd(input: &str) -> Result<ScmpCmd, Errno> {
                 }
 
                 let cap = Capability::from_str(cap)?;
+
+                // Reject ioctl which is for requests not paths.
+                if cap.contains(Capability::CAP_IOCTL) {
+                    return Err(Errno::EINVAL);
+                }
+
                 filter.insert(cap);
             }
 
@@ -739,6 +804,22 @@ fn host_parser(input: &str) -> IResult<&str, &str> {
     recognize(all_consuming(alias_tuple)).parse(input)
 }
 
+/// Converts a string representation of a number into a `i64` value.
+///
+/// The string can be in hexadecimal (prefixed with "0x"), octal
+/// (prefixed with "0o"), or decimal format. If the conversion fails, it
+/// returns an `Errno::EINVAL` error.
+pub fn str2i64(value: &[u8]) -> Result<i64, Errno> {
+    if is_prefix(value, b"0x") || is_prefix(value, b"0X") {
+        btoi_radix::<i64>(&value[2..], 16)
+    } else if is_prefix(value, b"0o") || is_prefix(value, b"0O") {
+        btoi_radix::<i64>(&value[2..], 8)
+    } else {
+        btoi::<i64>(value)
+    }
+    .or(Err(Errno::EINVAL))
+}
+
 /// Converts a string representation of a number into a `u64` value.
 ///
 /// The string can be in hexadecimal (prefixed with "0x"), octal
@@ -755,6 +836,30 @@ pub fn str2u64(value: &[u8]) -> Result<u64, Errno> {
     .or(Err(Errno::EINVAL))
 }
 
+/// Converts a string representation of a duration to a `Duration` value.
+///
+/// The string may be human formatted in which case it will be parsed by
+/// the dur crate.
+pub fn str2secs(value: &str) -> Result<Duration, Errno> {
+    if let Ok(value) = str2u64(value.as_bytes()) {
+        Ok(Duration::from_secs(value.into()))
+    } else {
+        value.parse::<Duration>().or(Err(Errno::EINVAL))
+    }
+}
+
+/// Converts a string representation of a duration to a `Duration` value.
+///
+/// The string may be human formatted in which case it will be parsed by
+/// the dur crate.
+pub fn str2micros(value: &str) -> Result<Duration, Errno> {
+    if let Ok(value) = str2u64(value.as_bytes()) {
+        Ok(Duration::from_micros(value.into()))
+    } else {
+        value.parse::<Duration>().or(Err(Errno::EINVAL))
+    }
+}
+
 /// Converts a string representation of a number into a `u32` value.
 ///
 /// The string can be in hexadecimal (prefixed with "0x"), octal
@@ -775,6 +880,129 @@ pub fn str2u32(value: &[u8]) -> Result<u32, Errno> {
 mod tests {
     use super::*;
 
+    #[test]
+    fn test_str2i64_decimal() {
+        assert_eq!(str2i64(b"0"), Ok(0));
+        assert_eq!(str2i64(b"123"), Ok(123));
+        assert_eq!(str2i64(b"-456"), Ok(-456));
+        assert_eq!(str2i64(b"9223372036854775807"), Ok(i64::MAX));
+        assert_eq!(str2i64(b"-9223372036854775808"), Ok(i64::MIN));
+    }
+
+    #[test]
+    fn test_str2i64_hexadecimal() {
+        assert_eq!(str2i64(b"0x0"), Ok(0));
+        assert_eq!(str2i64(b"0x1a"), Ok(26));
+        assert_eq!(str2i64(b"0xFF"), Ok(255));
+        assert_eq!(str2i64(b"0XDeAdBeEf"), Ok(0xDEADBEEF));
+    }
+
+    #[test]
+    fn test_str2i64_octal() {
+        assert_eq!(str2i64(b"0o0"), Ok(0));
+        assert_eq!(str2i64(b"0o17"), Ok(15));
+        assert_eq!(str2i64(b"0o777"), Ok(511));
+        assert_eq!(str2i64(b"0O755"), Ok(493));
+    }
+
+    #[test]
+    fn test_str2i64_invalid() {
+        assert_eq!(str2i64(b""), Err(Errno::EINVAL));
+        assert_eq!(str2i64(b"abc"), Err(Errno::EINVAL));
+        assert_eq!(str2i64(b"12.34"), Err(Errno::EINVAL));
+    }
+
+    #[test]
+    fn test_str2u64_decimal() {
+        assert_eq!(str2u64(b"0"), Ok(0));
+        assert_eq!(str2u64(b"123"), Ok(123));
+        assert_eq!(str2u64(b"18446744073709551615"), Ok(u64::MAX));
+    }
+
+    #[test]
+    fn test_str2u64_hexadecimal() {
+        assert_eq!(str2u64(b"0x0"), Ok(0));
+        assert_eq!(str2u64(b"0xdeadbeef"), Ok(0xDEADBEEF));
+        assert_eq!(str2u64(b"0XCAFEBABE"), Ok(0xCAFEBABE));
+    }
+
+    #[test]
+    fn test_str2u64_octal() {
+        assert_eq!(str2u64(b"0o0"), Ok(0));
+        assert_eq!(str2u64(b"0o777"), Ok(511));
+        assert_eq!(str2u64(b"0O644"), Ok(420));
+    }
+
+    #[test]
+    fn test_str2u64_invalid() {
+        assert_eq!(str2u64(b""), Err(Errno::EINVAL));
+        assert_eq!(str2u64(b"-1"), Err(Errno::EINVAL));
+        assert_eq!(str2u64(b"abc"), Err(Errno::EINVAL));
+    }
+
+    #[test]
+    fn test_str2u32_decimal() {
+        assert_eq!(str2u32(b"0"), Ok(0));
+        assert_eq!(str2u32(b"123"), Ok(123));
+        assert_eq!(str2u32(b"4294967295"), Ok(u32::MAX));
+    }
+
+    #[test]
+    fn test_str2u32_hexadecimal() {
+        assert_eq!(str2u32(b"0x0"), Ok(0));
+        assert_eq!(str2u32(b"0xFFFFFFFF"), Ok(u32::MAX));
+        assert_eq!(str2u32(b"0xcafe"), Ok(0xCAFE));
+    }
+
+    #[test]
+    fn test_str2u32_octal() {
+        assert_eq!(str2u32(b"0o0"), Ok(0));
+        assert_eq!(str2u32(b"0o777"), Ok(511));
+    }
+
+    #[test]
+    fn test_str2u32_invalid() {
+        assert_eq!(str2u32(b""), Err(Errno::EINVAL));
+        assert_eq!(str2u32(b"-1"), Err(Errno::EINVAL));
+        assert_eq!(str2u32(b"4294967296"), Err(Errno::EINVAL)); // overflow
+    }
+
+    #[test]
+    fn test_str2secs_numeric() {
+        assert_eq!(str2secs("0"), Ok(Duration::from_secs(0)));
+        assert_eq!(str2secs("60"), Ok(Duration::from_secs(60)));
+        assert_eq!(str2secs("3600"), Ok(Duration::from_secs(3600)));
+    }
+
+    #[test]
+    fn test_str2secs_hex() {
+        assert_eq!(str2secs("0x3c"), Ok(Duration::from_secs(60)));
+    }
+
+    #[test]
+    fn test_str2secs_invalid() {
+        assert_eq!(str2secs(""), Err(Errno::EINVAL));
+        assert_eq!(str2secs("abc"), Err(Errno::EINVAL));
+    }
+
+    #[test]
+    fn test_str2micros_numeric() {
+        assert_eq!(str2micros("0"), Ok(Duration::from_micros(0)));
+        assert_eq!(str2micros("1000"), Ok(Duration::from_micros(1000)));
+        assert_eq!(str2micros("1000000"), Ok(Duration::from_micros(1_000_000)));
+    }
+
+    #[test]
+    fn test_str2micros_hex() {
+        assert_eq!(str2micros("0x3e8"), Ok(Duration::from_micros(1000)));
+    }
+
+    #[test]
+    fn test_str2micros_invalid() {
+        assert_eq!(str2micros(""), Err(Errno::EINVAL));
+        assert_eq!(str2micros("abc"), Err(Errno::EINVAL));
+    }
+
     #[test]
     fn test_parse_bind_parse_bind_plus_root_readonly() {
         let cmd = "bind+/:/:ro";
@@ -782,7 +1010,7 @@ mod tests {
         assert_eq!(bc.op, '+');
         assert_eq!(bc.src, "/");
         assert_eq!(bc.dst, "/");
-        assert!(bc.opt.contains(MsFlags::MS_RDONLY));
+        assert!(bc.opt.contains(MountAttrFlags::MOUNT_ATTR_RDONLY));
         assert!(bc.dat.is_none());
     }
 
@@ -809,7 +1037,7 @@ mod tests {
         assert!(bc.opt.is_empty());
         assert_eq!(
             bc.dat.unwrap(),
-            XPathBuf::from("lowerdir=/tmp/lower,upperdir=/tmp/upper,workdir=/tmp/work")
+            "lowerdir=/tmp/lower,upperdir=/tmp/upper,workdir=/tmp/work",
         );
     }
 
@@ -820,10 +1048,10 @@ mod tests {
         assert_eq!(bc.op, '+');
         assert_eq!(bc.src, "tmpfs");
         assert_eq!(bc.dst, "/tmp");
-        assert!(bc.opt.contains(MsFlags::MS_RDONLY));
-        assert!(bc.opt.contains(MsFlags::MS_NOSUID));
+        assert!(bc.opt.contains(MountAttrFlags::MOUNT_ATTR_RDONLY));
+        assert!(bc.opt.contains(MountAttrFlags::MOUNT_ATTR_NOSUID));
         // "size=10M" is unknown -> goes into dat_buf
-        assert_eq!(bc.dat.unwrap(), XPathBuf::from("size=10M"));
+        assert_eq!(bc.dat.unwrap(), "size=10M");
     }
 
     #[test]
@@ -1388,6 +1616,101 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_parse_scmp_parse_all_x() {
+        let cmd = parse_scmp_cmd("allow/all-x+/home/***").unwrap();
+        assert_eq!(
+            cmd,
+            ScmpCmd {
+                action: Action::Allow,
+                filter: Capability::CAP_ALL_X,
+                op: '+',
+                pat: ScmpPattern::Path("/home/***".into()),
+            }
+        );
+    }
+
+    #[test]
+    fn test_parse_scmp_parse_all_x_minus_op() {
+        let cmd = parse_scmp_cmd("deny/all-x-/tmp/***").unwrap();
+        assert_eq!(
+            cmd,
+            ScmpCmd {
+                action: Action::Deny,
+                filter: Capability::CAP_ALL_X,
+                op: '-',
+                pat: ScmpPattern::Path("/tmp/***".into()),
+            }
+        );
+    }
+
+    #[test]
+    fn test_parse_scmp_parse_all_x_caret_op() {
+        let cmd = parse_scmp_cmd("filter/all-x^/var/***").unwrap();
+        assert_eq!(
+            cmd,
+            ScmpCmd {
+                action: Action::Filter,
+                filter: Capability::CAP_ALL_X,
+                op: '^',
+                pat: ScmpPattern::Path("/var/***".into()),
+            }
+        );
+    }
+
+    #[test]
+    fn test_parse_scmp_parse_all_x_with_read() {
+        let cmd = parse_scmp_cmd("allow/all-x,read+/data").unwrap();
+        assert_eq!(
+            cmd,
+            ScmpCmd {
+                action: Action::Allow,
+                filter: Capability::CAP_ALL_X,
+                op: '+',
+                pat: ScmpPattern::Path("/data".into()),
+            }
+        );
+    }
+
+    #[test]
+    fn test_parse_scmp_parse_read_with_all_x() {
+        let cmd = parse_scmp_cmd("warn/read,all-x-/secure").unwrap();
+        assert_eq!(
+            cmd,
+            ScmpCmd {
+                action: Action::Warn,
+                filter: Capability::CAP_ALL_X,
+                op: '-',
+                pat: ScmpPattern::Path("/secure".into()),
+            }
+        );
+    }
+
+    #[test]
+    fn test_parse_scmp_parse_all_x_with_exec() {
+        // all-x combined with exec should give CAP_GLOB (since all-x = all - exec)
+        let cmd = parse_scmp_cmd("allow/all-x,exec+/bin").unwrap();
+        assert_eq!(
+            cmd,
+            ScmpCmd {
+                action: Action::Allow,
+                filter: Capability::CAP_GLOB,
+                op: '+',
+                pat: ScmpPattern::Path("/bin".into()),
+            }
+        );
+    }
+
+    #[test]
+    fn test_parse_scmp_parse_all_vs_all_x_distinction() {
+        // Ensure `all` is correctly parsed when not followed by `-x`
+        let cmd = parse_scmp_cmd("allow/all+/path").unwrap();
+        assert_eq!(cmd.filter, Capability::CAP_GLOB);
+
+        let cmd = parse_scmp_cmd("allow/all-x+/path").unwrap();
+        assert_eq!(cmd.filter, Capability::CAP_ALL_X);
+    }
+
     #[test]
     fn test_parse_scmp_parse_all_with_others() {
         let cmd = parse_scmp_cmd("deny/all,read+/tmp").unwrap();
@@ -1412,7 +1735,7 @@ mod tests {
             }
         );
 
-        let cmd = parse_scmp_cmd("filter/ioctl,all,chdir^/tmp").unwrap();
+        let cmd = parse_scmp_cmd("filter/all,chdir^/tmp").unwrap();
         assert_eq!(
             cmd,
             ScmpCmd {
@@ -1702,4 +2025,109 @@ mod tests {
         assert_eq!(parse_netlink_cmd("allow/net/link+"), Err(Errno::EINVAL));
         assert_eq!(parse_netlink_cmd("allow/net/link-"), Err(Errno::EINVAL));
     }
+
+    #[test]
+    fn test_parse_port_range_single_port() {
+        assert_eq!(parse_port_range("80"), Ok(80..=80));
+        assert_eq!(parse_port_range("443"), Ok(443..=443));
+        assert_eq!(parse_port_range("0"), Ok(0..=0));
+        assert_eq!(parse_port_range("65535"), Ok(65535..=65535));
+    }
+
+    #[test]
+    fn test_parse_port_range_range() {
+        assert_eq!(parse_port_range("1024-65535"), Ok(1024..=65535));
+        assert_eq!(parse_port_range("80-443"), Ok(80..=443));
+        assert_eq!(parse_port_range("0-1023"), Ok(0..=1023));
+    }
+
+    #[test]
+    fn test_parse_port_range_reversed_normalizes() {
+        // Reversed range should be auto-normalized
+        assert_eq!(parse_port_range("100-50"), Ok(50..=100));
+        assert_eq!(parse_port_range("65535-1024"), Ok(1024..=65535));
+    }
+
+    #[test]
+    fn test_parse_port_range_same_port() {
+        assert_eq!(parse_port_range("8080-8080"), Ok(8080..=8080));
+    }
+
+    #[test]
+    fn test_parse_port_range_invalid() {
+        assert_eq!(parse_port_range(""), Err(Errno::EINVAL));
+        assert_eq!(parse_port_range("abc"), Err(Errno::EINVAL));
+        assert_eq!(parse_port_range("-1"), Err(Errno::EINVAL));
+        assert_eq!(parse_port_range("65536"), Err(Errno::EINVAL));
+        assert_eq!(parse_port_range("80-abc"), Err(Errno::EINVAL));
+        assert_eq!(parse_port_range("abc-80"), Err(Errno::EINVAL));
+    }
+
+    #[test]
+    fn test_parse_fs_cmd_allow_plus() {
+        let cmd = parse_fs_cmd("allow/fs+ext4").unwrap();
+        assert_eq!(cmd.action, Action::Allow);
+        assert_eq!(cmd.op, '+');
+        assert_eq!(cmd.fs_type, "ext4");
+    }
+
+    #[test]
+    fn test_parse_fs_cmd_deny_minus() {
+        let cmd = parse_fs_cmd("deny/fs-tmpfs").unwrap();
+        assert_eq!(cmd.action, Action::Deny);
+        assert_eq!(cmd.op, '-');
+        assert_eq!(cmd.fs_type, "tmpfs");
+    }
+
+    #[test]
+    fn test_parse_fs_cmd_filter_caret() {
+        let cmd = parse_fs_cmd("filter/fs^btrfs").unwrap();
+        assert_eq!(cmd.action, Action::Filter);
+        assert_eq!(cmd.op, '^');
+        assert_eq!(cmd.fs_type, "btrfs");
+    }
+
+    #[test]
+    fn test_parse_fs_cmd_all_actions() {
+        let actions = [
+            ("allow", Action::Allow),
+            ("deny", Action::Deny),
+            ("filter", Action::Filter),
+            ("warn", Action::Warn),
+            ("stop", Action::Stop),
+            ("abort", Action::Abort),
+            ("kill", Action::Kill),
+            ("panic", Action::Panic),
+            ("exit", Action::Exit),
+        ];
+        for (act_str, act_enum) in actions {
+            let input = format!("{}/fs+xfs", act_str);
+            let cmd = parse_fs_cmd(&input).unwrap();
+            assert_eq!(cmd.action, act_enum);
+        }
+    }
+
+    #[test]
+    fn test_parse_fs_cmd_invalid_action() {
+        assert_eq!(parse_fs_cmd("block/fs+ext4"), Err(Errno::EINVAL));
+        assert_eq!(parse_fs_cmd("reject/fs-tmpfs"), Err(Errno::EINVAL));
+    }
+
+    #[test]
+    fn test_parse_fs_cmd_missing_fs_type() {
+        assert_eq!(parse_fs_cmd("allow/fs+"), Err(Errno::EINVAL));
+        assert_eq!(parse_fs_cmd("deny/fs-"), Err(Errno::EINVAL));
+    }
+
+    #[test]
+    fn test_parse_fs_cmd_invalid_operator() {
+        assert_eq!(parse_fs_cmd("allow/fs*ext4"), Err(Errno::EINVAL));
+        assert_eq!(parse_fs_cmd("allow/fs=tmpfs"), Err(Errno::EINVAL));
+    }
+
+    #[test]
+    fn test_parse_fs_cmd_missing_prefix() {
+        assert_eq!(parse_fs_cmd("allow+ext4"), Err(Errno::EINVAL));
+        assert_eq!(parse_fs_cmd("fs+ext4"), Err(Errno::EINVAL));
+    }
 }
