@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/ptrace.rs: Utilities for ptrace(2)
 //
-// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 // Based in part upon strace which is:
 //   Copyright (c) 2016-2021 The strace developers.
 //   SPDX-License-Identifier: LGPL-2.1-or-later
@@ -248,9 +248,11 @@ pub fn ptrace_skip_syscall(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<
 
     #[cfg(target_arch = "x86_64")]
     {
-        use libc::{c_void, iovec, ptrace, NT_PRSTATUS, PTRACE_GETREGSET, PTRACE_SETREGSET};
+        use libc::{c_void, iovec, ptrace, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::{SCMP_ARCH_X32, SCMP_ARCH_X86, SCMP_ARCH_X86_64};
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure the architecture matches.
         if !matches!(arch, SCMP_ARCH_X86_64 | SCMP_ARCH_X86 | SCMP_ARCH_X32) {
             return Err(Errno::EINVAL);
@@ -336,6 +338,8 @@ pub fn ptrace_skip_syscall(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<
         use libc::{c_void, iovec, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::SCMP_ARCH_RISCV64;
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure we're working with the correct architecture.
         if arch != SCMP_ARCH_RISCV64 {
             return Err(Errno::EINVAL);
@@ -352,7 +356,7 @@ pub fn ptrace_skip_syscall(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<
 
         // SAFETY: Retrieve the current register state.
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         // Modify the syscall number (a7 holds the syscall number on RISC-V)
@@ -362,10 +366,8 @@ pub fn ptrace_skip_syscall(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<
         regs.a0 = (-(errno.map(|err| err as i32).unwrap_or(0) as i64)) as u64;
 
         // SAFETY: Set the modified register state.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &io) })
+            .map(drop)
     }
 
     #[cfg(not(any(
@@ -398,9 +400,11 @@ pub fn ptrace_skip_syscall(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<
 pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<(), Errno> {
     #[cfg(target_arch = "x86_64")]
     {
-        use libc::{c_void, iovec, ptrace, NT_PRSTATUS, PTRACE_GETREGSET, PTRACE_SETREGSET};
+        use libc::{c_void, iovec, ptrace, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::{SCMP_ARCH_X32, SCMP_ARCH_X86, SCMP_ARCH_X86_64};
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure the architecture matches.
         if !matches!(arch, SCMP_ARCH_X86_64 | SCMP_ARCH_X86 | SCMP_ARCH_X32) {
             return Err(Errno::EINVAL);
@@ -473,6 +477,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         use libc::{c_void, iovec, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::{SCMP_ARCH_AARCH64, SCMP_ARCH_ARM};
 
+        use crate::compat::NT_PRSTATUS;
+
         // Define the arm_pt_regs for arm as described in the system headers
         #[repr(C)]
         #[derive(Copy, Clone)]
@@ -500,7 +506,7 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
 
         // SAFETY: Retrieve the current register state
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         match arch {
@@ -540,10 +546,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         }
 
         // SAFETY: Write the modified register state back.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })
+            .map(drop)
     }
 
     #[cfg(target_arch = "arm")]
@@ -552,6 +556,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         use libseccomp_sys::SCMP_ARCH_ARM;
         use nix::errno::Errno;
 
+        use crate::compat::NT_PRSTATUS;
+
         // Define the ARM register structure.
         #[repr(C)]
         struct ArmPtRegs {
@@ -574,7 +580,7 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
 
         // SAFETY: Retrieve the current register state
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         // Modify the return value in R0.
@@ -591,10 +597,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         }
 
         // SAFETY: Write the modified register state back.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })
+            .map(drop)
     }
 
     #[cfg(target_arch = "m68k")]
@@ -732,6 +736,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         use libc::{c_void, iovec, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::SCMP_ARCH_RISCV64;
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure we're working with the correct architecture.
         if arch != SCMP_ARCH_RISCV64 {
             return Err(Errno::EINVAL);
@@ -748,7 +754,7 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
 
         // SAFETY: Retrieve the current register state
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         #[expect(clippy::arithmetic_side_effects)]
@@ -765,10 +771,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         }
 
         // SAFETY: Write the modified register state back
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })
+            .map(drop)
     }
 
     #[cfg(target_arch = "s390x")]
@@ -776,6 +780,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         use libc::{c_void, iovec, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::SCMP_ARCH_S390X;
 
+        use crate::compat::NT_PRSTATUS;
+
         #[repr(C, align(8))]
         struct psw_t {
             mask: u64,
@@ -806,7 +812,7 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
 
         // SAFETY: Retrieve the current register state.
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         #[expect(clippy::arithmetic_side_effects)]
@@ -823,10 +829,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         }
 
         // SAFETY: Write the modified register state back.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &io) })
+            .map(drop)
     }
 
     #[cfg(target_arch = "powerpc")]
@@ -899,6 +903,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         use libc::{c_void, iovec, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::{SCMP_ARCH_PPC, SCMP_ARCH_PPC64, SCMP_ARCH_PPC64LE};
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure we're working with the correct architecture.
         if !matches!(arch, SCMP_ARCH_PPC | SCMP_ARCH_PPC64 | SCMP_ARCH_PPC64LE) {
             return Err(Errno::EINVAL);
@@ -917,7 +923,7 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
 
         // SAFETY: Retrieve the current register state
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         match arch {
@@ -975,10 +981,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         }
 
         // SAFETY: Write the modified register state back.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })
+            .map(drop)
     }
 
     #[cfg(target_arch = "loongarch64")]
@@ -986,6 +990,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         use libc::{c_void, iovec, user_regs_struct, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::SCMP_ARCH_LOONGARCH64;
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure we're working with the correct architecture.
         if arch != SCMP_ARCH_LOONGARCH64 {
             return Err(Errno::EINVAL);
@@ -1002,7 +1008,7 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
 
         // SAFETY: Retrieve the current register state
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         #[expect(clippy::arithmetic_side_effects)]
@@ -1019,10 +1025,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         }
 
         // SAFETY: Write the modified register state back.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })
+            .map(drop)
     }
 
     #[cfg(not(any(
@@ -1052,9 +1056,11 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
 pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
     #[cfg(target_arch = "x86_64")]
     {
-        use libc::{c_void, iovec, ptrace, NT_PRSTATUS, PTRACE_GETREGSET};
+        use libc::{c_void, iovec, ptrace, PTRACE_GETREGSET};
         use libseccomp_sys::{SCMP_ARCH_X32, SCMP_ARCH_X86, SCMP_ARCH_X86_64};
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure the architecture matches.
         if !matches!(arch, SCMP_ARCH_X86_64 | SCMP_ARCH_X86 | SCMP_ARCH_X32) {
             return Err(Errno::EINVAL);
@@ -1124,6 +1130,8 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         use libc::{c_void, iovec, PTRACE_GETREGSET};
         use libseccomp_sys::{SCMP_ARCH_AARCH64, SCMP_ARCH_ARM};
 
+        use crate::compat::NT_PRSTATUS;
+
         // ARM user regs struct
         #[repr(C)]
         #[derive(Copy, Clone)]
@@ -1149,7 +1157,7 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
 
         // Get registers
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         let val: i64 = match arch {
@@ -1179,6 +1187,8 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         use libc::{c_void, iovec, PTRACE_GETREGSET};
         use libseccomp_sys::SCMP_ARCH_ARM;
 
+        use crate::compat::NT_PRSTATUS;
+
         #[repr(C)]
         struct ArmPtRegs {
             uregs: [u32; 18],
@@ -1197,7 +1207,7 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
 
         // Get registers
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         let val = (regs.uregs[0] as i32) as i64;
@@ -1299,6 +1309,8 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         use libc::{c_void, iovec, PTRACE_GETREGSET};
         use libseccomp_sys::SCMP_ARCH_RISCV64;
 
+        use crate::compat::NT_PRSTATUS;
+
         if arch != SCMP_ARCH_RISCV64 {
             return Err(Errno::EINVAL);
         }
@@ -1310,7 +1322,7 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         };
 
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         let val = regs.a0 as i64;
@@ -1326,6 +1338,8 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         use libc::{c_void, iovec, PTRACE_GETREGSET};
         use libseccomp_sys::SCMP_ARCH_S390X;
 
+        use crate::compat::NT_PRSTATUS;
+
         #[repr(C, align(8))]
         struct psw_t {
             mask: u64,
@@ -1351,7 +1365,7 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         };
 
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         // Syscall return value is in gprs[2]
@@ -1425,6 +1439,8 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         use libc::{c_void, iovec, PTRACE_GETREGSET};
         use libseccomp_sys::{SCMP_ARCH_PPC, SCMP_ARCH_PPC64, SCMP_ARCH_PPC64LE};
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure we're working with the correct architecture.
         if !matches!(arch, SCMP_ARCH_PPC | SCMP_ARCH_PPC64 | SCMP_ARCH_PPC64LE) {
             return Err(Errno::EINVAL);
@@ -1443,7 +1459,7 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
 
         // SAFETY: Retrieve the current register state
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         // On PPC, the return value is always in gpr[3].
@@ -1501,6 +1517,8 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         use libc::{c_void, iovec, user_regs_struct, PTRACE_GETREGSET};
         use libseccomp_sys::SCMP_ARCH_LOONGARCH64;
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure we're working with the correct architecture.
         if arch != SCMP_ARCH_LOONGARCH64 {
             return Err(Errno::EINVAL);
@@ -1513,7 +1531,7 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         };
 
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         let val = regs.regs[4] as i64;
@@ -1553,10 +1571,10 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
 pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno> {
     #[cfg(target_arch = "x86_64")]
     {
-        use libc::{c_void, iovec, ptrace, NT_PRSTATUS, PTRACE_GETREGSET, PTRACE_SETREGSET};
+        use libc::{c_void, iovec, ptrace, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::{SCMP_ARCH_X32, SCMP_ARCH_X86, SCMP_ARCH_X86_64};
 
-        use crate::confine::X32_SYSCALL_BIT;
+        use crate::{compat::NT_PRSTATUS, confine::X32_SYSCALL_BIT};
 
         // Ensure the architecture matches.
         if !matches!(arch, SCMP_ARCH_X86_64 | SCMP_ARCH_X86 | SCMP_ARCH_X32) {
@@ -1704,6 +1722,8 @@ pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno>
         use libc::{c_void, iovec, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::SCMP_ARCH_RISCV64;
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure we're working with the correct architecture.
         if arch != SCMP_ARCH_RISCV64 {
             return Err(Errno::EINVAL);
@@ -1720,7 +1740,7 @@ pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno>
 
         // SAFETY: Retrieve the current register state.
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         // Modify the syscall number (a7 holds the syscall number on RISC-V)
@@ -1730,10 +1750,8 @@ pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno>
         regs.a0 = (-(Errno::ENOSYS as i64)) as u64;
 
         // SAFETY: Set the modified register state.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &io) })
+            .map(drop)
     }
 
     #[cfg(target_arch = "s390x")]
@@ -1741,6 +1759,8 @@ pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno>
         use libc::{c_void, iovec, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::SCMP_ARCH_S390X;
 
+        use crate::compat::NT_PRSTATUS;
+
         #[repr(C, align(8))]
         struct psw_t {
             mask: u64,
@@ -1771,17 +1791,15 @@ pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno>
 
         // SAFETY: Retrieve the current registers.
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         // Set the syscall number in GPR2.
         regs.gprs[2] = sysno;
 
         // SAFETY: Update the registers with the new syscall number.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &io) })
+            .map(drop)
     }
 
     #[cfg(target_arch = "loongarch64")]
@@ -1789,6 +1807,8 @@ pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno>
         use libc::{c_void, iovec, user_regs_struct, PTRACE_GETREGSET, PTRACE_SETREGSET};
         use libseccomp_sys::SCMP_ARCH_LOONGARCH64;
 
+        use crate::compat::NT_PRSTATUS;
+
         // Ensure we're working with the correct architecture.
         if arch != SCMP_ARCH_LOONGARCH64 {
             return Err(Errno::EINVAL);
@@ -1805,17 +1825,15 @@ pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno>
 
         // SAFETY: Retrieve the current register state.
         Errno::result(unsafe {
-            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
+            libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io)
         })?;
 
         // Modify the syscall number (regs[11] holds the syscall number on LOONGARCH64)
         regs.regs[11] = sysno;
 
         // SAFETY: Set the modified register state.
-        Errno::result(unsafe {
-            libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &io)
-        })
-        .map(drop)
+        Errno::result(unsafe { libc::ptrace(PTRACE_SETREGSET, pid.as_raw(), NT_PRSTATUS, &io) })
+            .map(drop)
     }
 
     #[cfg(target_arch = "m68k")]
