@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/sealbox.rs: SealBox<T> for type-safe memory sealing/protecting
 //
-// Copyright (c) 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2025, 2026 Ali Polatel <alip@chesswob.org>
 // Big thanks and respect to cehteh in ##rust for the invaluable ideas
 // leading to the eventual implementation of this library.
 //
@@ -29,19 +29,17 @@ use libc::c_void;
 use libseccomp::ScmpSyscall;
 use nix::{
     errno::Errno,
-    sys::{
-        mman::{mmap_anonymous, mprotect, munmap, MapFlags, ProtFlags},
-        prctl::set_vma_anon_name,
-    },
+    sys::mman::{mmap_anonymous, mprotect, munmap, MapFlags, ProtFlags},
     unistd::{sysconf, SysconfVar},
 };
 
 use crate::{
+    compat::set_vma_anon_name,
     config::HAVE_MADV_GUARD,
     hash::{SydHashMap, SydIndexMap, SydRandomState},
 };
 
-// Note mseal may not be available,
+// Note mseal(2) may not be available,
 // and libc::SYS_mseal may not be defined.
 // Therefore we query the number using libseccomp.
 static SYS_MSEAL: LazyLock<Option<libc::c_long>> = LazyLock::new(|| {
@@ -76,7 +74,7 @@ pub fn mseal(addr: NonNull<c_void>, len: NonZeroUsize) -> Result<(), Errno> {
     let num = SYS_MSEAL.ok_or(Errno::ENOSYS)?;
 
     // SAFETY: In libc we trust.
-    Errno::result(unsafe { libc::syscall(num, addr, len.get(), 0) }).map(drop)
+    Errno::result(unsafe { libc::syscall(num, addr.as_ptr(), len.get(), 0) }).map(drop)
 }
 
 /// Safe wrapper for mprotect(2) with PROT_NONE.
@@ -91,6 +89,12 @@ pub fn mprotect_readonly(addr: NonNull<c_void>, len: NonZeroUsize) -> Result<(),
     unsafe { mprotect(addr, len.get(), ProtFlags::PROT_READ) }
 }
 
+/// Safe wrapper for mprotect(2) with PROT_EXEC.
+pub fn mprotect_xonly(addr: NonNull<c_void>, len: NonZeroUsize) -> Result<(), Errno> {
+    // SAFETY: In libc we trust.
+    unsafe { mprotect(addr, len.get(), ProtFlags::PROT_EXEC) }
+}
+
 // Libc does not yet define MADV_GUARD_{INSTALL,REMOVE}.
 const MADV_GUARD_INSTALL: i32 = 102;
 const MADV_GUARD_REMOVE: i32 = 103;
@@ -107,8 +111,8 @@ pub fn madvise_guard_remove(addr: NonNull<c_void>, len: NonZeroUsize) -> Result<
     Errno::result(unsafe { libc::madvise(addr.as_ptr(), len.get(), MADV_GUARD_REMOVE) }).map(drop)
 }
 
-// Get page size.
-fn getpagesize() -> Result<usize, Errno> {
+/// Get the system page size.
+pub fn getpagesize() -> Result<usize, Errno> {
     sysconf(SysconfVar::PAGE_SIZE)?
         .ok_or(Errno::EINVAL)?
         .try_into()
@@ -169,7 +173,7 @@ impl<T> SealBox<T> {
             madvise_guard_install(self.map_ptr, self.map_len)?;
         }
 
-        // Set VMA name as necessary.
+        // Set VMA name in debug mode.
         // Note, this may return EINVAL if CONFIG_ANON_VMA_NAME is not set,
         // therefore we ignore errors here.
         if vma_name.is_some() {
@@ -396,7 +400,7 @@ impl<E> SealBoxSlice<E> {
             madvise_guard_install(self.map_ptr, self.map_len)?;
         }
 
-        // Set VMA name as necessary.
+        // Set VMA name in debug mode.
         // Note, this may return EINVAL if CONFIG_ANON_VMA_NAME is not set,
         // therefore we ignore errors here.
         if vma_name.is_some() {
@@ -645,12 +649,12 @@ pub enum SealableVec<T> {
 impl<T> SealableVec<T> {
     /// Create from an existing `Vec<T>`.
     pub fn new(vec: Vec<T>) -> Self {
-        SealableVec::Unsealed(vec)
+        Self::Unsealed(vec)
     }
 
     /// Create with a specified capacity.
     pub fn with_capacity(cap: usize) -> Self {
-        SealableVec::Unsealed(Vec::with_capacity(cap))
+        Self::Unsealed(Vec::with_capacity(cap))
     }
 
     /// Seal into a read-only mmap, returning a new sealed container.
@@ -659,9 +663,9 @@ impl<T> SealableVec<T> {
         T: Clone,
     {
         match self {
-            SealableVec::Unsealed(v) => {
+            Self::Unsealed(v) => {
                 let sealed = SealBoxSlice::from_slice(&v)?.seal(vma_name)?;
-                Ok(SealableVec::Sealed(sealed))
+                Ok(Self::Sealed(sealed))
             }
             _ => Err(Errno::EPERM),
         }
@@ -670,8 +674,8 @@ impl<T> SealableVec<T> {
     /// Return the number of elements.
     pub fn len(&self) -> usize {
         match self {
-            SealableVec::Unsealed(v) => v.len(),
-            SealableVec::Sealed(s) => s.len(),
+            Self::Unsealed(v) => v.len(),
+            Self::Sealed(s) => s.len(),
         }
     }
 
@@ -683,16 +687,16 @@ impl<T> SealableVec<T> {
     /// Return the current capacity (sealed==len).
     pub fn capacity(&self) -> usize {
         match self {
-            SealableVec::Unsealed(v) => v.capacity(),
-            SealableVec::Sealed(s) => s.len(),
+            Self::Unsealed(v) => v.capacity(),
+            Self::Sealed(s) => s.len(),
         }
     }
 
     /// Get a reference to the element at `idx`.
     pub fn get(&self, idx: usize) -> Option<&T> {
         match self {
-            SealableVec::Unsealed(v) => v.get(idx),
-            SealableVec::Sealed(s) => s.get(idx),
+            Self::Unsealed(v) => v.get(idx),
+            Self::Sealed(s) => s.get(idx),
         }
     }
 
@@ -725,8 +729,8 @@ impl<T> SealableVec<T> {
     /// Return an iterator over all elements.
     pub fn iter(&self) -> std::slice::Iter<'_, T> {
         match self {
-            SealableVec::Unsealed(v) => v.iter(),
-            SealableVec::Sealed(s) => s.iter(),
+            Self::Unsealed(v) => v.iter(),
+            Self::Sealed(s) => s.iter(),
         }
     }
 
@@ -738,44 +742,44 @@ impl<T> SealableVec<T> {
     /// Push `value` if unsealed, else return `Err(EPERM)`.
     pub fn push(&mut self, value: T) -> Result<(), Errno> {
         match self {
-            SealableVec::Unsealed(v) => {
+            Self::Unsealed(v) => {
                 v.push(value);
                 Ok(())
             }
-            SealableVec::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
     /// Pop the last element if unsealed, else return `None`.
     pub fn pop(&mut self) -> Option<T> {
         match self {
-            SealableVec::Unsealed(v) => v.pop(),
-            SealableVec::Sealed(_) => None,
+            Self::Unsealed(v) => v.pop(),
+            Self::Sealed(_) => None,
         }
     }
 
     /// Insert at `index` if unsealed, else return `Err(EPERM)`.
     pub fn insert(&mut self, index: usize, element: T) -> Result<(), Errno> {
         match self {
-            SealableVec::Unsealed(v) => {
+            Self::Unsealed(v) => {
                 v.insert(index, element);
                 Ok(())
             }
-            SealableVec::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
     /// Remove and return element at `index` if unsealed, else return `None`.
     pub fn remove(&mut self, index: usize) -> Option<T> {
         match self {
-            SealableVec::Unsealed(v) => Some(v.remove(index)),
-            SealableVec::Sealed(_) => None,
+            Self::Unsealed(v) => Some(v.remove(index)),
+            Self::Sealed(_) => None,
         }
     }
 
     /// Clear all elements if unsealed.
     pub fn clear(&mut self) {
-        if let SealableVec::Unsealed(v) = self {
+        if let Self::Unsealed(v) = self {
             v.clear();
         }
     }
@@ -783,41 +787,41 @@ impl<T> SealableVec<T> {
     /// Reserve capacity for `additional` more elements if unsealed.
     pub fn reserve(&mut self, additional: usize) -> Result<(), Errno> {
         match self {
-            SealableVec::Unsealed(v) => {
+            Self::Unsealed(v) => {
                 v.reserve(additional);
                 Ok(())
             }
-            SealableVec::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
     /// Shrink to fit if unsealed.
     pub fn shrink_to_fit(&mut self) -> Result<(), Errno> {
         match self {
-            SealableVec::Unsealed(v) => {
+            Self::Unsealed(v) => {
                 v.shrink_to_fit();
                 Ok(())
             }
-            SealableVec::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
     /// Append `other` Vec if unsealed, else return `Err(EPERM)`.
     pub fn append(&mut self, other: &mut Vec<T>) -> Result<(), Errno> {
         match self {
-            SealableVec::Unsealed(v) => {
+            Self::Unsealed(v) => {
                 v.append(other);
                 Ok(())
             }
-            SealableVec::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
     /// Split off at `at` if unsealed, else return `Err(EPERM)`.
     pub fn split_off(&mut self, at: usize) -> Result<Vec<T>, Errno> {
         match self {
-            SealableVec::Unsealed(v) => Ok(v.split_off(at)),
-            SealableVec::Sealed(_) => Err(Errno::EPERM),
+            Self::Unsealed(v) => Ok(v.split_off(at)),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
@@ -827,11 +831,11 @@ impl<T> SealableVec<T> {
         F: FnMut(&T) -> bool,
     {
         match self {
-            SealableVec::Unsealed(v) => {
+            Self::Unsealed(v) => {
                 v.retain(f);
                 Ok(())
             }
-            SealableVec::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
@@ -841,22 +845,22 @@ impl<T> SealableVec<T> {
         F: FnMut(&mut T) -> bool,
     {
         match self {
-            SealableVec::Unsealed(v) => {
+            Self::Unsealed(v) => {
                 v.retain_mut(f);
                 Ok(())
             }
-            SealableVec::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
     /// Extend from `iter` if unsealed, else return `Err(EPERM)`.
     pub fn extend<I: IntoIterator<Item = T>>(&mut self, iter: I) -> Result<(), Errno> {
         match self {
-            SealableVec::Unsealed(v) => {
+            Self::Unsealed(v) => {
                 v.extend(iter);
                 Ok(())
             }
-            SealableVec::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
@@ -866,8 +870,8 @@ impl<T> SealableVec<T> {
         R: std::ops::RangeBounds<usize>,
     {
         match self {
-            SealableVec::Unsealed(v) => Ok(v.drain(range)),
-            SealableVec::Sealed(_) => Err(Errno::EPERM),
+            Self::Unsealed(v) => Ok(v.drain(range)),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 }
@@ -876,8 +880,8 @@ impl<T> Deref for SealableVec<T> {
     type Target = [T];
     fn deref(&self) -> &Self::Target {
         match self {
-            SealableVec::Unsealed(v) => v.as_slice(),
-            SealableVec::Sealed(s) => s.deref(),
+            Self::Unsealed(v) => v.as_slice(),
+            Self::Sealed(s) => s.deref(),
         }
     }
 }
@@ -885,8 +889,8 @@ impl<T> Deref for SealableVec<T> {
 impl<T> DerefMut for SealableVec<T> {
     fn deref_mut(&mut self) -> &mut Self::Target {
         match self {
-            SealableVec::Unsealed(v) => v.as_mut_slice(),
-            SealableVec::Sealed(_) => panic!("cannot mutably borrow a sealed Vec"),
+            Self::Unsealed(v) => v.as_mut_slice(),
+            Self::Sealed(_) => panic!("cannot mutably borrow a sealed Vec"),
         }
     }
 }
@@ -914,7 +918,7 @@ impl<T: Eq> Eq for SealableVec<T> {}
 
 impl<T> Default for SealableVec<T> {
     fn default() -> Self {
-        SealableVec::Unsealed(Vec::new())
+        Self::Unsealed(Vec::new())
     }
 }
 
@@ -941,12 +945,12 @@ pub enum SealableVecDeque<T> {
 impl<T> SealableVecDeque<T> {
     /// Create from an existing `VecDeque<T>`.
     pub fn new(dq: VecDeque<T>) -> Self {
-        SealableVecDeque::Unsealed(dq)
+        Self::Unsealed(dq)
     }
 
     /// Create with a specified capacity.
     pub fn with_capacity(cap: usize) -> Self {
-        SealableVecDeque::Unsealed(VecDeque::with_capacity(cap))
+        Self::Unsealed(VecDeque::with_capacity(cap))
     }
 
     /// Seal into a read-only mmap, returning a new sealed container.
@@ -955,9 +959,9 @@ impl<T> SealableVecDeque<T> {
         T: Clone,
     {
         match self {
-            SealableVecDeque::Unsealed(dq) => {
+            Self::Unsealed(dq) => {
                 let sealed = SealBoxSlice::new_deque(dq)?.seal(vma_name)?;
-                Ok(SealableVecDeque::Sealed(sealed))
+                Ok(Self::Sealed(sealed))
             }
             _ => Err(Errno::EPERM),
         }
@@ -966,8 +970,8 @@ impl<T> SealableVecDeque<T> {
     /// Return the number of elements.
     pub fn len(&self) -> usize {
         match self {
-            SealableVecDeque::Unsealed(dq) => dq.len(),
-            SealableVecDeque::Sealed(s) => s.len(),
+            Self::Unsealed(dq) => dq.len(),
+            Self::Sealed(s) => s.len(),
         }
     }
 
@@ -979,16 +983,16 @@ impl<T> SealableVecDeque<T> {
     /// Return the current capacity (sealed==len).
     pub fn capacity(&self) -> usize {
         match self {
-            SealableVecDeque::Unsealed(dq) => dq.capacity(),
-            SealableVecDeque::Sealed(s) => s.len(),
+            Self::Unsealed(dq) => dq.capacity(),
+            Self::Sealed(s) => s.len(),
         }
     }
 
     /// Get a reference to the element at `idx`.
     pub fn get(&self, idx: usize) -> Option<&T> {
         match self {
-            SealableVecDeque::Unsealed(dq) => dq.get(idx),
-            SealableVecDeque::Sealed(s) => s.get(idx),
+            Self::Unsealed(dq) => dq.get(idx),
+            Self::Sealed(s) => s.get(idx),
         }
     }
 
@@ -1013,71 +1017,71 @@ impl<T> SealableVecDeque<T> {
     /// Return an iterator over all elements (linearizing head+tail for unsealed).
     pub fn iter(&self) -> std::iter::Chain<std::slice::Iter<'_, T>, std::slice::Iter<'_, T>> {
         match self {
-            SealableVecDeque::Unsealed(dq) => {
+            Self::Unsealed(dq) => {
                 let (head, tail) = dq.as_slices();
                 head.iter().chain(tail.iter())
             }
-            SealableVecDeque::Sealed(s) => s.iter().chain([].iter()),
+            Self::Sealed(s) => s.iter().chain([].iter()),
         }
     }
 
     /// Return the two contiguous slices (tail, head) or entire slice if sealed.
     pub fn as_slices(&self) -> (&[T], &[T]) {
         match self {
-            SealableVecDeque::Unsealed(dq) => dq.as_slices(),
-            SealableVecDeque::Sealed(s) => (s.deref(), &[]),
+            Self::Unsealed(dq) => dq.as_slices(),
+            Self::Sealed(s) => (s.deref(), &[]),
         }
     }
 
     /// Push `elem` to the back if unsealed, else return `Err(EPERM)`.
     pub fn push_back(&mut self, elem: T) -> Result<(), Errno> {
         match self {
-            SealableVecDeque::Unsealed(dq) => {
+            Self::Unsealed(dq) => {
                 dq.push_back(elem);
                 Ok(())
             }
-            SealableVecDeque::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
     /// Push `elem` to the front if unsealed, else return `Err(EPERM)`.
     pub fn push_front(&mut self, elem: T) -> Result<(), Errno> {
         match self {
-            SealableVecDeque::Unsealed(dq) => {
+            Self::Unsealed(dq) => {
                 dq.push_front(elem);
                 Ok(())
             }
-            SealableVecDeque::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
     /// Pop from the back if unsealed, else return `None`.
     pub fn pop_back(&mut self) -> Option<T> {
         match self {
-            SealableVecDeque::Unsealed(dq) => dq.pop_back(),
-            SealableVecDeque::Sealed(_) => None,
+            Self::Unsealed(dq) => dq.pop_back(),
+            Self::Sealed(_) => None,
         }
     }
 
     /// Pop from the front if unsealed, else return `None`.
     pub fn pop_front(&mut self) -> Option<T> {
         match self {
-            SealableVecDeque::Unsealed(dq) => dq.pop_front(),
-            SealableVecDeque::Sealed(_) => None,
+            Self::Unsealed(dq) => dq.pop_front(),
+            Self::Sealed(_) => None,
         }
     }
 
     /// Remove and return element at `index` if unsealed, else return `None`.
     pub fn remove(&mut self, index: usize) -> Option<T> {
         match self {
-            SealableVecDeque::Unsealed(v) => v.remove(index),
-            SealableVecDeque::Sealed(_) => None,
+            Self::Unsealed(v) => v.remove(index),
+            Self::Sealed(_) => None,
         }
     }
 
     /// Clear all elements if unsealed.
     pub fn clear(&mut self) {
-        if let SealableVecDeque::Unsealed(dq) = self {
+        if let Self::Unsealed(dq) = self {
             dq.clear();
         }
     }
@@ -1085,33 +1089,33 @@ impl<T> SealableVecDeque<T> {
     /// Reserve capacity for `additional` more elements if unsealed.
     pub fn reserve(&mut self, additional: usize) -> Result<(), Errno> {
         match self {
-            SealableVecDeque::Unsealed(dq) => {
+            Self::Unsealed(dq) => {
                 dq.reserve(additional);
                 Ok(())
             }
-            SealableVecDeque::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
     /// Rearranges the internal storage of this deque so it is one contiguous slice.
     pub fn make_contiguous(&mut self) -> Result<(), Errno> {
         match self {
-            SealableVecDeque::Unsealed(dq) => {
+            Self::Unsealed(dq) => {
                 dq.make_contiguous();
                 Ok(())
             }
-            SealableVecDeque::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
     /// Shrink to fit if unsealed.
     pub fn shrink_to_fit(&mut self) -> Result<(), Errno> {
         match self {
-            SealableVecDeque::Unsealed(dq) => {
+            Self::Unsealed(dq) => {
                 dq.shrink_to_fit();
                 Ok(())
             }
-            SealableVecDeque::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
@@ -1121,11 +1125,11 @@ impl<T> SealableVecDeque<T> {
         F: FnMut(&T) -> bool,
     {
         match self {
-            SealableVecDeque::Unsealed(v) => {
+            Self::Unsealed(v) => {
                 v.retain(f);
                 Ok(())
             }
-            SealableVecDeque::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
@@ -1135,22 +1139,22 @@ impl<T> SealableVecDeque<T> {
         F: FnMut(&mut T) -> bool,
     {
         match self {
-            SealableVecDeque::Unsealed(v) => {
+            Self::Unsealed(v) => {
                 v.retain_mut(f);
                 Ok(())
             }
-            SealableVecDeque::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
     /// Extend from `iter` if unsealed, else return `Err(EPERM)`.
     pub fn extend<I: IntoIterator<Item = T>>(&mut self, iter: I) -> Result<(), Errno> {
         match self {
-            SealableVecDeque::Unsealed(v) => {
+            Self::Unsealed(v) => {
                 v.extend(iter);
                 Ok(())
             }
-            SealableVecDeque::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
@@ -1160,8 +1164,8 @@ impl<T> SealableVecDeque<T> {
         R: std::ops::RangeBounds<usize>,
     {
         match self {
-            SealableVecDeque::Unsealed(v) => Ok(v.drain(range)),
-            SealableVecDeque::Sealed(_) => Err(Errno::EPERM),
+            Self::Unsealed(v) => Ok(v.drain(range)),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 }
@@ -1170,11 +1174,11 @@ impl<T> Deref for SealableVecDeque<T> {
     type Target = [T];
     fn deref(&self) -> &Self::Target {
         match self {
-            SealableVecDeque::Unsealed(dq) => {
+            Self::Unsealed(dq) => {
                 let (head, _) = dq.as_slices();
                 head
             }
-            SealableVecDeque::Sealed(s) => s.deref(),
+            Self::Sealed(s) => s.deref(),
         }
     }
 }
@@ -1182,11 +1186,11 @@ impl<T> Deref for SealableVecDeque<T> {
 impl<T> DerefMut for SealableVecDeque<T> {
     fn deref_mut(&mut self) -> &mut Self::Target {
         match self {
-            SealableVecDeque::Unsealed(dq) => {
+            Self::Unsealed(dq) => {
                 let (head, _) = dq.as_mut_slices();
                 head
             }
-            SealableVecDeque::Sealed(_) => panic!("cannot mutably borrow a sealed VecDeque"),
+            Self::Sealed(_) => panic!("cannot mutably borrow a sealed VecDeque"),
         }
     }
 }
@@ -1214,7 +1218,7 @@ impl<T: Eq> Eq for SealableVecDeque<T> {}
 
 impl<T> Default for SealableVecDeque<T> {
     fn default() -> Self {
-        SealableVecDeque::Unsealed(VecDeque::new())
+        Self::Unsealed(VecDeque::new())
     }
 }
 
@@ -1226,7 +1230,7 @@ impl<T: fmt::Debug> fmt::Debug for SealableVecDeque<T> {
 
 impl<T> From<VecDeque<T>> for SealableVecDeque<T> {
     fn from(dq: VecDeque<T>) -> Self {
-        SealableVecDeque::Unsealed(dq)
+        Self::Unsealed(dq)
     }
 }
 
@@ -1261,12 +1265,12 @@ pub enum SealableHashMap<K, V> {
 impl<K, V> SealableHashMap<K, V> {
     /// Create from an existing `HashMap<K, V>`.
     pub fn new(map: HashMap<K, V>) -> Self {
-        SealableHashMap::Unsealed(map)
+        Self::Unsealed(map)
     }
 
     /// Create with a specified capacity.
     pub fn with_capacity(cap: usize) -> Self {
-        SealableHashMap::Unsealed(HashMap::with_capacity(cap))
+        Self::Unsealed(HashMap::with_capacity(cap))
     }
 
     /// Seal into a read-only mmap of sorted `(K, V)` pairs.
@@ -1278,19 +1282,19 @@ impl<K, V> SealableHashMap<K, V> {
         V: Clone,
     {
         match self {
-            SealableHashMap::Unsealed(map) => {
+            Self::Unsealed(map) => {
                 let sealed = SealBoxSlice::from_hashmap(map)?.seal(vma_name)?;
-                Ok(SealableHashMap::Sealed(sealed))
+                Ok(Self::Sealed(sealed))
             }
-            SealableHashMap::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
     /// Returns the number of elements.
     pub fn len(&self) -> usize {
         match self {
-            SealableHashMap::Unsealed(m) => m.len(),
-            SealableHashMap::Sealed(s) => s.len(),
+            Self::Unsealed(m) => m.len(),
+            Self::Sealed(s) => s.len(),
         }
     }
 
@@ -1308,8 +1312,8 @@ impl<K, V> SealableHashMap<K, V> {
         Q: Ord + Eq + Hash + ?Sized,
     {
         match self {
-            SealableHashMap::Unsealed(m) => m.get(key),
-            SealableHashMap::Sealed(s) => s
+            Self::Unsealed(m) => m.get(key),
+            Self::Sealed(s) => s
                 .binary_search_by(|(k, _)| k.borrow().cmp(key))
                 .ok()
                 .map(|idx| &s[idx].1),
@@ -1334,8 +1338,8 @@ impl<K, V> SealableHashMap<K, V> {
         K: Eq + Hash,
     {
         match self {
-            SealableHashMap::Unsealed(m) => Ok(m.insert(key, value)),
-            SealableHashMap::Sealed(_) => Err(Errno::EPERM),
+            Self::Unsealed(m) => Ok(m.insert(key, value)),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
@@ -1348,14 +1352,14 @@ impl<K, V> SealableHashMap<K, V> {
         Q: Eq + Hash + ?Sized,
     {
         match self {
-            SealableHashMap::Unsealed(m) => m.remove(key),
-            SealableHashMap::Sealed(_) => None,
+            Self::Unsealed(m) => m.remove(key),
+            Self::Sealed(_) => None,
         }
     }
 
     /// Clears the map if unsealed; does nothing if sealed.
     pub fn clear(&mut self) {
-        if let SealableHashMap::Unsealed(m) = self {
+        if let Self::Unsealed(m) = self {
             m.clear();
         }
     }
@@ -1368,11 +1372,11 @@ impl<K, V> SealableHashMap<K, V> {
         K: Eq + Hash,
     {
         match self {
-            SealableHashMap::Unsealed(m) => {
+            Self::Unsealed(m) => {
                 m.reserve(additional);
                 Ok(())
             }
-            SealableHashMap::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
@@ -1380,8 +1384,8 @@ impl<K, V> SealableHashMap<K, V> {
     /// for sealed, the number of entries.
     pub fn capacity(&self) -> usize {
         match self {
-            SealableHashMap::Unsealed(m) => m.capacity(),
-            SealableHashMap::Sealed(s) => s.len(),
+            Self::Unsealed(m) => m.capacity(),
+            Self::Sealed(s) => s.len(),
         }
     }
 
@@ -1391,26 +1395,26 @@ impl<K, V> SealableHashMap<K, V> {
         F: FnMut(&K, &mut V) -> bool,
     {
         match self {
-            SealableHashMap::Unsealed(m) => {
+            Self::Unsealed(m) => {
                 m.retain(f);
                 Ok(())
             }
-            SealableHashMap::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 }
 
 impl<K, V> Default for SealableHashMap<K, V> {
     fn default() -> Self {
-        SealableHashMap::Unsealed(HashMap::new())
+        Self::Unsealed(HashMap::new())
     }
 }
 
 impl<K: fmt::Debug, V: fmt::Debug> fmt::Debug for SealableHashMap<K, V> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            SealableHashMap::Unsealed(m) => m.fmt(f),
-            SealableHashMap::Sealed(s) => {
+            Self::Unsealed(m) => m.fmt(f),
+            Self::Sealed(s) => {
                 let mut dm = f.debug_map();
                 for pair in s.iter() {
                     dm.entry(&pair.0, &pair.1);
@@ -1427,14 +1431,14 @@ impl<K: Ord + Eq + Hash, V: PartialEq> PartialEq for SealableHashMap<K, V> {
             return false;
         }
         match self {
-            SealableHashMap::Unsealed(m) => {
+            Self::Unsealed(m) => {
                 for (k, v) in m {
                     if other.get(k) != Some(v) {
                         return false;
                     }
                 }
             }
-            SealableHashMap::Sealed(s) => {
+            Self::Sealed(s) => {
                 for (k, v) in s.iter() {
                     if other.get(k) != Some(v) {
                         return false;
@@ -1450,7 +1454,7 @@ impl<K: Ord + Eq + Hash, V: Eq> Eq for SealableHashMap<K, V> {}
 
 impl<K, V> From<HashMap<K, V>> for SealableHashMap<K, V> {
     fn from(m: HashMap<K, V>) -> Self {
-        SealableHashMap::Unsealed(m)
+        Self::Unsealed(m)
     }
 }
 
@@ -1480,12 +1484,12 @@ pub enum SealableSydHashMap<K, V> {
 impl<K, V> SealableSydHashMap<K, V> {
     /// Create from an existing `SydHashMap<K, V>`.
     pub fn new(map: SydHashMap<K, V>) -> Self {
-        SealableSydHashMap::Unsealed(map)
+        Self::Unsealed(map)
     }
 
     /// Create with a specified capacity.
     pub fn with_capacity(cap: usize) -> Self {
-        SealableSydHashMap::Unsealed(SydHashMap::with_capacity(cap))
+        Self::Unsealed(SydHashMap::with_capacity(cap))
     }
 
     /// Seal into a read-only mmap of sorted `(K, V)` pairs.
@@ -1497,19 +1501,19 @@ impl<K, V> SealableSydHashMap<K, V> {
         V: Clone,
     {
         match self {
-            SealableSydHashMap::Unsealed(map) => {
+            Self::Unsealed(map) => {
                 let sealed = SealBoxSlice::from_sydhashmap(map)?.seal(vma_name)?;
-                Ok(SealableSydHashMap::Sealed(sealed))
+                Ok(Self::Sealed(sealed))
             }
-            SealableSydHashMap::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
     /// Returns the number of elements.
     pub fn len(&self) -> usize {
         match self {
-            SealableSydHashMap::Unsealed(m) => m.len(),
-            SealableSydHashMap::Sealed(s) => s.len(),
+            Self::Unsealed(m) => m.len(),
+            Self::Sealed(s) => s.len(),
         }
     }
 
@@ -1527,8 +1531,8 @@ impl<K, V> SealableSydHashMap<K, V> {
         Q: Ord + Eq + Hash + ?Sized,
     {
         match self {
-            SealableSydHashMap::Unsealed(m) => m.get(key),
-            SealableSydHashMap::Sealed(s) => s
+            Self::Unsealed(m) => m.get(key),
+            Self::Sealed(s) => s
                 .binary_search_by(|(k, _)| k.borrow().cmp(key))
                 .ok()
                 .map(|idx| &s[idx].1),
@@ -1553,8 +1557,8 @@ impl<K, V> SealableSydHashMap<K, V> {
         K: Eq + Hash,
     {
         match self {
-            SealableSydHashMap::Unsealed(m) => Ok(m.insert(key, value)),
-            SealableSydHashMap::Sealed(_) => Err(Errno::EPERM),
+            Self::Unsealed(m) => Ok(m.insert(key, value)),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
@@ -1567,14 +1571,14 @@ impl<K, V> SealableSydHashMap<K, V> {
         Q: Eq + Hash + ?Sized,
     {
         match self {
-            SealableSydHashMap::Unsealed(m) => m.remove(key),
-            SealableSydHashMap::Sealed(_) => None,
+            Self::Unsealed(m) => m.remove(key),
+            Self::Sealed(_) => None,
         }
     }
 
     /// Clears the map if unsealed; does nothing if sealed.
     pub fn clear(&mut self) {
-        if let SealableSydHashMap::Unsealed(m) = self {
+        if let Self::Unsealed(m) = self {
             m.clear();
         }
     }
@@ -1587,11 +1591,11 @@ impl<K, V> SealableSydHashMap<K, V> {
         K: Eq + Hash,
     {
         match self {
-            SealableSydHashMap::Unsealed(m) => {
+            Self::Unsealed(m) => {
                 m.reserve(additional);
                 Ok(())
             }
-            SealableSydHashMap::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
@@ -1599,8 +1603,8 @@ impl<K, V> SealableSydHashMap<K, V> {
     /// for sealed, the number of entries.
     pub fn capacity(&self) -> usize {
         match self {
-            SealableSydHashMap::Unsealed(m) => m.capacity(),
-            SealableSydHashMap::Sealed(s) => s.len(),
+            Self::Unsealed(m) => m.capacity(),
+            Self::Sealed(s) => s.len(),
         }
     }
 
@@ -1610,26 +1614,26 @@ impl<K, V> SealableSydHashMap<K, V> {
         F: FnMut(&K, &mut V) -> bool,
     {
         match self {
-            SealableSydHashMap::Unsealed(m) => {
+            Self::Unsealed(m) => {
                 m.retain(f);
                 Ok(())
             }
-            SealableSydHashMap::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 }
 
 impl<K, V> Default for SealableSydHashMap<K, V> {
     fn default() -> Self {
-        SealableSydHashMap::Unsealed(SydHashMap::new())
+        Self::Unsealed(SydHashMap::new())
     }
 }
 
 impl<K: fmt::Debug, V: fmt::Debug> fmt::Debug for SealableSydHashMap<K, V> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            SealableSydHashMap::Unsealed(m) => m.fmt(f),
-            SealableSydHashMap::Sealed(s) => {
+            Self::Unsealed(m) => m.fmt(f),
+            Self::Sealed(s) => {
                 let mut dm = f.debug_map();
                 for pair in s.iter() {
                     dm.entry(&pair.0, &pair.1);
@@ -1646,14 +1650,14 @@ impl<K: Ord + Eq + Hash, V: PartialEq> PartialEq for SealableSydHashMap<K, V> {
             return false;
         }
         match self {
-            SealableSydHashMap::Unsealed(m) => {
+            Self::Unsealed(m) => {
                 for (k, v) in m {
                     if other.get(k) != Some(v) {
                         return false;
                     }
                 }
             }
-            SealableSydHashMap::Sealed(s) => {
+            Self::Sealed(s) => {
                 for (k, v) in s.iter() {
                     if other.get(k) != Some(v) {
                         return false;
@@ -1669,7 +1673,7 @@ impl<K: Ord + Eq + Hash, V: Eq> Eq for SealableSydHashMap<K, V> {}
 
 impl<K, V> From<SydHashMap<K, V>> for SealableSydHashMap<K, V> {
     fn from(m: SydHashMap<K, V>) -> Self {
-        SealableSydHashMap::Unsealed(m)
+        Self::Unsealed(m)
     }
 }
 
@@ -1699,12 +1703,12 @@ pub enum SealableSydIndexMap<K, V> {
 impl<K, V> SealableSydIndexMap<K, V> {
     /// Create from an existing `SydIndexMap<K, V>`.
     pub fn new(map: SydIndexMap<K, V>) -> Self {
-        SealableSydIndexMap::Unsealed(map)
+        Self::Unsealed(map)
     }
 
     /// Create with a specified capacity.
     pub fn with_capacity(cap: usize) -> Self {
-        SealableSydIndexMap::Unsealed(SydIndexMap::with_capacity_and_hasher(
+        Self::Unsealed(SydIndexMap::with_capacity_and_hasher(
             cap,
             SydRandomState::new(),
         ))
@@ -1719,19 +1723,19 @@ impl<K, V> SealableSydIndexMap<K, V> {
         V: Clone,
     {
         match self {
-            SealableSydIndexMap::Unsealed(map) => {
+            Self::Unsealed(map) => {
                 let sealed = SealBoxSlice::from_sydindexmap(map)?.seal(vma_name)?;
-                Ok(SealableSydIndexMap::Sealed(sealed))
+                Ok(Self::Sealed(sealed))
             }
-            SealableSydIndexMap::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
     /// Returns the number of elements.
     pub fn len(&self) -> usize {
         match self {
-            SealableSydIndexMap::Unsealed(m) => m.len(),
-            SealableSydIndexMap::Sealed(s) => s.len(),
+            Self::Unsealed(m) => m.len(),
+            Self::Sealed(s) => s.len(),
         }
     }
 
@@ -1749,8 +1753,8 @@ impl<K, V> SealableSydIndexMap<K, V> {
         Q: Ord + Eq + Hash + ?Sized,
     {
         match self {
-            SealableSydIndexMap::Unsealed(m) => m.get(key),
-            SealableSydIndexMap::Sealed(s) => s
+            Self::Unsealed(m) => m.get(key),
+            Self::Sealed(s) => s
                 .binary_search_by(|(k, _)| k.borrow().cmp(key))
                 .ok()
                 .map(|idx| &s[idx].1),
@@ -1775,8 +1779,8 @@ impl<K, V> SealableSydIndexMap<K, V> {
         K: Eq + Hash,
     {
         match self {
-            SealableSydIndexMap::Unsealed(m) => Ok(m.insert(key, value)),
-            SealableSydIndexMap::Sealed(_) => Err(Errno::EPERM),
+            Self::Unsealed(m) => Ok(m.insert(key, value)),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
@@ -1789,14 +1793,14 @@ impl<K, V> SealableSydIndexMap<K, V> {
         Q: Eq + Hash + ?Sized,
     {
         match self {
-            SealableSydIndexMap::Unsealed(m) => m.shift_remove(key),
-            SealableSydIndexMap::Sealed(_) => None,
+            Self::Unsealed(m) => m.shift_remove(key),
+            Self::Sealed(_) => None,
         }
     }
 
     /// Clears the map if unsealed; does nothing if sealed.
     pub fn clear(&mut self) {
-        if let SealableSydIndexMap::Unsealed(m) = self {
+        if let Self::Unsealed(m) = self {
             m.clear();
         }
     }
@@ -1809,11 +1813,11 @@ impl<K, V> SealableSydIndexMap<K, V> {
         K: Eq + Hash,
     {
         match self {
-            SealableSydIndexMap::Unsealed(m) => {
+            Self::Unsealed(m) => {
                 m.reserve(additional);
                 Ok(())
             }
-            SealableSydIndexMap::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
@@ -1821,8 +1825,8 @@ impl<K, V> SealableSydIndexMap<K, V> {
     /// for sealed, the number of entries.
     pub fn capacity(&self) -> usize {
         match self {
-            SealableSydIndexMap::Unsealed(m) => m.capacity(),
-            SealableSydIndexMap::Sealed(s) => s.len(),
+            Self::Unsealed(m) => m.capacity(),
+            Self::Sealed(s) => s.len(),
         }
     }
 
@@ -1832,26 +1836,26 @@ impl<K, V> SealableSydIndexMap<K, V> {
         F: FnMut(&K, &mut V) -> bool,
     {
         match self {
-            SealableSydIndexMap::Unsealed(m) => {
+            Self::Unsealed(m) => {
                 m.retain(f);
                 Ok(())
             }
-            SealableSydIndexMap::Sealed(_) => Err(Errno::EPERM),
+            Self::Sealed(_) => Err(Errno::EPERM),
         }
     }
 }
 
 impl<K, V> Default for SealableSydIndexMap<K, V> {
     fn default() -> Self {
-        SealableSydIndexMap::Unsealed(SydIndexMap::default())
+        Self::Unsealed(SydIndexMap::default())
     }
 }
 
 impl<K: fmt::Debug, V: fmt::Debug> fmt::Debug for SealableSydIndexMap<K, V> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            SealableSydIndexMap::Unsealed(m) => m.fmt(f),
-            SealableSydIndexMap::Sealed(s) => {
+            Self::Unsealed(m) => m.fmt(f),
+            Self::Sealed(s) => {
                 let mut dm = f.debug_map();
                 for pair in s.iter() {
                     dm.entry(&pair.0, &pair.1);
@@ -1868,14 +1872,14 @@ impl<K: Ord + Eq + Hash, V: PartialEq> PartialEq for SealableSydIndexMap<K, V> {
             return false;
         }
         match self {
-            SealableSydIndexMap::Unsealed(m) => {
+            Self::Unsealed(m) => {
                 for (k, v) in m {
                     if other.get(k) != Some(v) {
                         return false;
                     }
                 }
             }
-            SealableSydIndexMap::Sealed(s) => {
+            Self::Sealed(s) => {
                 for (k, v) in s.iter() {
                     if other.get(k) != Some(v) {
                         return false;
@@ -1891,7 +1895,7 @@ impl<K: Ord + Eq + Hash, V: Eq> Eq for SealableSydIndexMap<K, V> {}
 
 impl<K, V> From<SydIndexMap<K, V>> for SealableSydIndexMap<K, V> {
     fn from(m: SydIndexMap<K, V>) -> Self {
-        SealableSydIndexMap::Unsealed(m)
+        Self::Unsealed(m)
     }
 }
 
