@@ -799,22 +799,7 @@ macro_rules! skip_unless_cap {
 #[macro_export]
 macro_rules! skip_if_landlock_abi_supported {
     ($abi:expr) => {{
-        use std::process::Command;
-
-        // Run `syd-lock -A` to get the ABI version as exit code.
-        let status = Command::new(&*SYD_LOCK)
-            .arg("-A")
-            .stdin(Stdio::null())
-            .stdout(Stdio::null())
-            .status()
-            .expect("Failed to run syd-lock -A");
-
-        // SAFETY: We expect `syd-lock -A` to exit with a code.
-        let exit_code = status
-            .code()
-            .expect("Failed to get exit code from syd-lock -A");
-
-        if exit_code >= $abi {
+        if syd::landlock::ABI::new_current() >= syd::landlock::ABI::from($abi) {
             eprintln!(
                 "Test is for Landlock ABI versions older than {}, skipping!",
                 $abi
@@ -833,22 +818,7 @@ macro_rules! skip_if_landlock_abi_supported {
 #[macro_export]
 macro_rules! skip_unless_landlock_abi_supported {
     ($abi:expr) => {{
-        use std::process::Command;
-
-        // Run `syd-lock -A` to get the ABI version as exit code.
-        let status = Command::new(&*SYD_LOCK)
-            .arg("-A")
-            .stdin(Stdio::null())
-            .stdout(Stdio::null())
-            .status()
-            .expect("Failed to run syd-lock -A");
-
-        // SAFETY: We expect `syd-lock -A` to exit with a code.
-        let exit_code = status
-            .code()
-            .expect("Failed to get exit code from syd-lock -A");
-
-        if exit_code < $abi {
+        if syd::landlock::ABI::new_current() < syd::landlock::ABI::from($abi) {
             eprintln!("Test requires Landlock ABI version {}, skipping!", $abi);
             std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
             return Ok(());
