@@ -1,6 +1,6 @@
 use std::{ffi::OsStr, os::fd::RawFd};
 
-use nix::sys::signal::Signal;
+use nix::sys::{resource::rlim_t, signal::Signal};
 
 use crate::{
     sandbox::RawIoctlMap,
@@ -123,6 +123,18 @@ impl Command {
         self
     }
 
+    /// Enable kernel pointer in syscall arguments mitigations.
+    pub fn restrict_kptr(&mut self, restrict_kptr: bool) -> &mut Command {
+        self.config.restrict_kptr = restrict_kptr;
+        self
+    }
+
+    /// Hint whether prlimit(2) is enabled by seccomp(2) so resource limits can be applied late.
+    pub fn restrict_prlimit(&mut self, restrict_prlimit: bool) -> &mut Command {
+        self.config.restrict_prlimit = restrict_prlimit;
+        self
+    }
+
     /// Specify pseudoterminal file descriptor.
     pub fn pty(&mut self, fd: Option<RawFd>) -> &mut Command {
         self.pty_fd = fd;
@@ -134,4 +146,88 @@ impl Command {
         self.ioctl_denylist = denylist;
         self
     }
+
+    /// Specify RLIMIT_AS rlimit(2).
+    pub fn rlimit_as(&mut self, lim: Option<rlim_t>) -> &mut Command {
+        self.config.rlimit_as = lim;
+        self
+    }
+
+    /// Specify RLIMIT_CORE rlimit(2).
+    pub fn rlimit_core(&mut self, lim: Option<rlim_t>) -> &mut Command {
+        self.config.rlimit_core = lim;
+        self
+    }
+
+    /// Specify RLIMIT_CPU rlimit(2).
+    pub fn rlimit_cpu(&mut self, lim: Option<rlim_t>) -> &mut Command {
+        self.config.rlimit_cpu = lim;
+        self
+    }
+
+    /// Specify RLIMIT_DATA rlimit(2).
+    pub fn rlimit_data(&mut self, lim: Option<rlim_t>) -> &mut Command {
+        self.config.rlimit_data = lim;
+        self
+    }
+
+    /// Specify RLIMIT_FSIZE rlimit(2).
+    pub fn rlimit_fsize(&mut self, lim: Option<rlim_t>) -> &mut Command {
+        self.config.rlimit_fsize = lim;
+        self
+    }
+
+    /// Specify RLIMIT_MEMLOCK rlimit(2).
+    pub fn rlimit_memlock(&mut self, lim: Option<rlim_t>) -> &mut Command {
+        self.config.rlimit_memlock = lim;
+        self
+    }
+
+    /// Specify RLIMIT_MSGQUEUE rlimit(2).
+    pub fn rlimit_msgqueue(&mut self, lim: Option<rlim_t>) -> &mut Command {
+        self.config.rlimit_msgqueue = lim;
+        self
+    }
+
+    /// Specify RLIMIT_NICE rlimit(2).
+    pub fn rlimit_nice(&mut self, lim: Option<rlim_t>) -> &mut Command {
+        self.config.rlimit_nice = lim;
+        self
+    }
+
+    /// Specify RLIMIT_NOFILE rlimit(2).
+    pub fn rlimit_nofile(&mut self, lim: Option<rlim_t>) -> &mut Command {
+        self.config.rlimit_nofile = lim;
+        self
+    }
+
+    /// Specify RLIMIT_NPROC rlimit(2).
+    pub fn rlimit_nproc(&mut self, lim: Option<rlim_t>) -> &mut Command {
+        self.config.rlimit_nproc = lim;
+        self
+    }
+
+    /// Specify RLIMIT_RTPRIO rlimit(2).
+    pub fn rlimit_rtprio(&mut self, lim: Option<rlim_t>) -> &mut Command {
+        self.config.rlimit_rtprio = lim;
+        self
+    }
+
+    /// Specify RLIMIT_RTTIME rlimit(2).
+    pub fn rlimit_rttime(&mut self, lim: Option<rlim_t>) -> &mut Command {
+        self.config.rlimit_rttime = lim;
+        self
+    }
+
+    /// Specify RLIMIT_SIGPENDING rlimit(2).
+    pub fn rlimit_sigpending(&mut self, lim: Option<rlim_t>) -> &mut Command {
+        self.config.rlimit_sigpending = lim;
+        self
+    }
+
+    /// Specify RLIMIT_STACK rlimit(2).
+    pub fn rlimit_stack(&mut self, lim: Option<rlim_t>) -> &mut Command {
+        self.config.rlimit_stack = lim;
+        self
+    }
 }
