@@ -42,7 +42,6 @@ mod status;
 mod std_api;
 mod stdio;
 mod wait;
-mod zombies;
 
 use std::{ffi::CString, os::unix::io::RawFd};
 
@@ -57,7 +56,6 @@ pub use crate::unshare::{
     namespace::Namespace,
     status::ExitStatus,
     stdio::{Fd, Stdio},
-    zombies::{child_events, reap_zombies, ChildEvent},
 };
 
 /// Main class for running processes. Works in the spirit of builder pattern.
