@@ -9,8 +9,7 @@
 use std::{
     collections::VecDeque,
     os::unix::process::CommandExt,
-    process::{exit, Command, ExitCode, Stdio},
-    time::Duration,
+    process::{Command, ExitCode, Stdio},
 };
 
 use nix::{
@@ -19,12 +18,13 @@ use nix::{
     sys::stat::Mode,
     unistd::Pid,
 };
-use syd::{path::XPathBuf, proc::proc_environ_read, syslog::LogLevel, wordexp::WordExp};
+use syd::{path::XPathBuf, proc::proc_environ_read, syslog::LogLevel};
 
 // Set global allocator to GrapheneOS allocator.
 #[cfg(all(
     not(coverage),
     not(feature = "prof"),
+    not(target_os = "android"),
     target_page_size_4k,
     target_pointer_width = "64"
 ))]
@@ -49,20 +49,28 @@ syd::main! {
             return Ok(ExitCode::SUCCESS);
         }
         Some("-e") => {
-            let var = if let Some(var) = args.pop_front() {
+            let _var = if let Some(var) = args.pop_front() {
                 var
             } else {
                 eprintln!("Error: -e requires an argument!");
                 return Ok(ExitCode::FAILURE);
             };
-            match WordExp::expand(&var, true, Duration::from_secs(3)) {
+
+            #[cfg(target_os = "android")]
+            {
+                eprintln!("Error: -e is not supported on Android!");
+                return Ok(ExitCode::FAILURE);
+            }
+
+            #[cfg(not(target_os = "android"))]
+            match syd::wordexp::WordExp::expand(&_var, true, dur::Duration::from_secs(3)) {
                 Ok(val) => {
                     print!("{val}");
                     return Ok(ExitCode::SUCCESS);
                 }
                 Err(err) => {
                     eprintln!("Error: {err}");
-                    exit(err.into());
+                    std::process::exit(err.into());
                 }
             };
         }
