@@ -15,6 +15,22 @@ use std::{
 use nix::{errno::Errno, unistd::ttyname};
 use syd::err::SydResult;
 
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    not(target_os = "android"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
+#[global_allocator]
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 /// Resembles the `which` command, finds a program in PATH.
 fn which(command: &str, realpath: bool) -> SydResult<String> {
     let out = Command::new("which")
