@@ -11,6 +11,8 @@ use std::{os::unix::ffi::OsStrExt, process::ExitCode, str::FromStr, time::Durati
 
 use libc::c_int;
 use libseccomp::{ScmpArch, ScmpSyscall};
+#[expect(clippy::disallowed_types)]
+use nix::unistd::ForkResult;
 use nix::{
     errno::Errno,
     fcntl::OFlag,
@@ -19,7 +21,7 @@ use nix::{
         stat::lstat,
         wait::{waitpid, WaitPidFlag, WaitStatus},
     },
-    unistd::{fork, ForkResult, Pid},
+    unistd::{fork, Pid},
 };
 use syd::{
     config::proc_init_simple,
