@@ -48,12 +48,8 @@ use syd::{
         confine_mdwe, confine_rlimit, confine_rlimit_zero, confine_scmp_madvise, CLONE_NEWTIME,
     },
     cookie::{
-        safe_connect, safe_pipe2, safe_shutdown, safe_socket, safer_accept4, ACCEPT4_COOKIE_ARG4,
-        ACCEPT4_COOKIE_ARG5, CONNECT_COOKIE_ARG3, CONNECT_COOKIE_ARG4, CONNECT_COOKIE_ARG5,
-        PIPE2_COOKIE_ARG2, PIPE2_COOKIE_ARG3, PIPE2_COOKIE_ARG4, PIPE2_COOKIE_ARG5,
-        SHUTDOWN_COOKIE_ARG2, SHUTDOWN_COOKIE_ARG3, SHUTDOWN_COOKIE_ARG4, SHUTDOWN_COOKIE_ARG5,
-        SOCKET_COOKIE_ARG3, SOCKET_COOKIE_ARG4, SOCKET_COOKIE_ARG5, SYS_ACCEPT4, SYS_CONNECT,
-        SYS_SHUTDOWN, SYS_SOCKET,
+        safe_connect, safe_pipe2, safe_shutdown, safe_socket, safer_accept4, CookieIdx,
+        SYSCOOKIE_POOL, SYS_ACCEPT4, SYS_CONNECT, SYS_SHUTDOWN, SYS_SOCKET,
     },
     err::{SydError, SydResult},
     fd::closeexcept,
@@ -482,10 +478,10 @@ impl<'a> Proxy<'a> {
             ScmpSyscall::from_name("pipe2").unwrap(),
             &[
                 scmp_cmp!($arg1 == (OFlag::O_NONBLOCK | OFlag::O_CLOEXEC).bits() as u64),
-                scmp_cmp!($arg2 == (*PIPE2_COOKIE_ARG2).into()),
-                scmp_cmp!($arg3 == (*PIPE2_COOKIE_ARG3).into()),
-                scmp_cmp!($arg4 == (*PIPE2_COOKIE_ARG4).into()),
-                scmp_cmp!($arg5 == (*PIPE2_COOKIE_ARG5).into()),
+                scmp_cmp!($arg2 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg2).into()),
+                scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg3).into()),
+                scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg4).into()),
+                scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg5).into()),
             ],
         )?;
 
@@ -505,9 +501,9 @@ impl<'a> Proxy<'a> {
                     scmp_cmp!($arg0 == sock_domain),
                     scmp_cmp!($arg1 == sock_type),
                     scmp_cmp!($arg2 == sock_protocol),
-                    scmp_cmp!($arg3 == (*SOCKET_COOKIE_ARG3).into()),
-                    scmp_cmp!($arg4 == (*SOCKET_COOKIE_ARG4).into()),
-                    scmp_cmp!($arg5 == (*SOCKET_COOKIE_ARG5).into()),
+                    scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::SocketArg3).into()),
+                    scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::SocketArg4).into()),
+                    scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::SocketArg5).into()),
                 ],
             )?;
         } else {
@@ -525,9 +521,9 @@ impl<'a> Proxy<'a> {
                 &[
                     scmp_cmp!($arg1 == addr_ptr),
                     scmp_cmp!($arg2 == addr_len),
-                    scmp_cmp!($arg3 == (*CONNECT_COOKIE_ARG3).into()),
-                    scmp_cmp!($arg4 == (*CONNECT_COOKIE_ARG4).into()),
-                    scmp_cmp!($arg5 == (*CONNECT_COOKIE_ARG5).into()),
+                    scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::ConnectArg3).into()),
+                    scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::ConnectArg4).into()),
+                    scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::ConnectArg5).into()),
                 ],
             )?;
         } else {
@@ -550,8 +546,8 @@ impl<'a> Proxy<'a> {
                     scmp_cmp!($arg1 == 0u64), // struct sockaddr *_Nullable restrict addr
                     scmp_cmp!($arg2 == 0u64), // socklen_t *_Nullable restrict addrlen
                     scmp_cmp!($arg3 == (SockFlag::SOCK_NONBLOCK | SockFlag::SOCK_CLOEXEC).bits() as u64),
-                    scmp_cmp!($arg4 == (*ACCEPT4_COOKIE_ARG4).into()),
-                    scmp_cmp!($arg5 == (*ACCEPT4_COOKIE_ARG5).into()),
+                    scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::Accept4Arg4).into()),
+                    scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::Accept4Arg5).into()),
                 ],
             )?;
         } else {
@@ -572,10 +568,10 @@ impl<'a> Proxy<'a> {
                 syscall,
                 &[
                     scmp_cmp!($arg1 == libc::SHUT_WR as u64),
-                    scmp_cmp!($arg2 == (*SHUTDOWN_COOKIE_ARG2).into()),
-                    scmp_cmp!($arg3 == (*SHUTDOWN_COOKIE_ARG3).into()),
-                    scmp_cmp!($arg4 == (*SHUTDOWN_COOKIE_ARG4).into()),
-                    scmp_cmp!($arg5 == (*SHUTDOWN_COOKIE_ARG5).into()),
+                    scmp_cmp!($arg2 == SYSCOOKIE_POOL.get(CookieIdx::ShutdownArg2).into()),
+                    scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::ShutdownArg3).into()),
+                    scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::ShutdownArg4).into()),
+                    scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::ShutdownArg5).into()),
                 ],
             )?;
         } else {
