@@ -26,6 +26,7 @@ use libseccomp::{scmp_cmp, RawSyscall, ScmpAction, ScmpFilterContext, ScmpSyscal
 use nix::{
     errno::Errno,
     fcntl::{posix_fadvise, splice, tee, OFlag, PosixFadviseAdvice, SpliceFFlags},
+    sched::{unshare, CloneFlags},
     unistd::{lseek64, write, Gid, Uid, Whence},
 };
 use serde::{Serialize, Serializer};
@@ -33,6 +34,7 @@ use serde::{Serialize, Serializer};
 #[cfg(target_arch = "x86")]
 use crate::cookie::CookieIdx::Ftruncate64Arg3;
 use crate::{
+    alert,
     config::*,
     confine::{
         confine_scmp_clone, confine_scmp_clone3, confine_scmp_fadvise, confine_scmp_madvise,
@@ -48,6 +50,8 @@ use crate::{
         SydHashMap, BLOCK_SIZE, HMAC_TAG_SIZE, IV, IV_SIZE, SYD3_HDR_SIZE,
     },
     info,
+    landlock::Errata,
+    landlock_policy::LandlockPolicy,
     lookup::FileInfo,
     ofd::lock_fd,
     path::{XPath, XPathBuf},
@@ -158,7 +162,20 @@ impl AesWorker {
         flags: Flags,
         transit_uids: &[(Uid, Uid)],
         transit_gids: &[(Gid, Gid)],
+        dry_run: bool,
     ) -> SydResult<ScmpFilterContext> {
+        if !dry_run {
+            // SAFETY: Set up a Landlock sandbox to disallow all access.
+            let abi = crate::landlock::ABI::new_current();
+            let errata = crate::landlock::Errata::query();
+            let policy = LandlockPolicy {
+                scoped_abs: true,
+                scoped_sig: errata.contains(Errata::SCOPED_SIGNAL_SAME_TGID),
+                ..Default::default()
+            };
+            let _ = policy.restrict_self(abi);
+        }
+
         // Create seccomp filter with default action.
         let mut ctx = ScmpFilterContext::new(ScmpAction::KillProcess)?;
 
@@ -472,6 +489,18 @@ impl AesWorker {
             .name("syd_aes".to_string())
             .stack_size(AES_STACK_SIZE)
             .spawn(move || {
+                // SAFETY:
+                // 1. We use exit_group(2) here to bail, because this unsharing
+                //    is a critical safety feature.
+                // 2. CLONE_FILES cannot be unshared because this thread must share
+                //    file descriptors with the emulator threads.
+                if let Err(errno) = unshare(CloneFlags::CLONE_FS | CloneFlags::CLONE_SYSVSEM) {
+                    alert!("ctx": "boot", "op": "unshare_aes_thread",
+                        "msg": format!("failed to unshare(CLONE_FS|CLONE_SYSVSEM): {errno}"),
+                        "err": errno as i32);
+                    std::process::exit(101);
+                }
+
                 // Honour dry-run when exporting.
                 let dry_run =
                     env::var_os(ENV_SKIP_SCMP).is_some() || ExportMode::from_env().is_some();
@@ -480,8 +509,12 @@ impl AesWorker {
                     // Load the filter immediately.
                     // Logging is permitted as long as the `pid` key is unused.
                     // See prepare_confine for more information.
-                    let ctx =
-                        Self::prepare_confine(self.flags, &self.transit_uids, &self.transit_gids)?;
+                    let ctx = Self::prepare_confine(
+                        self.flags,
+                        &self.transit_uids,
+                        &self.transit_gids,
+                        false,
+                    )?;
                     ctx.load()?;
 
                     let safe_setid = self
