@@ -118,11 +118,12 @@ impl Worker {
                 // Unshare:
                 // 1. CLONE_FS so cwd and umask are per-thread.
                 // 2. CLONE_FILES so file descriptor table is per-thread.
+                // 3. CLONE_SYSVSEM so System V semaphores are per-thread.
                 //
                 // Note, we cannot do 2 if Crypt sanboxing is on because
                 // emulator threads have to share memory fds with AES
                 // threads.
-                let mut unshare_flags = CloneFlags::CLONE_FS;
+                let mut unshare_flags = CloneFlags::CLONE_FS | CloneFlags::CLONE_SYSVSEM;
                 let is_crypt = self.cache.crypt_map.is_some();
                 if !is_crypt {
                     unshare_flags.insert(CloneFlags::CLONE_FILES);
@@ -522,14 +523,14 @@ impl Worker {
             }
         }
 
-        // Allow unshare(2) with CLONE_FS|CLONE_FILES only.
+        // Allow unshare(2) with CLONE_FS|CLONE_FILES|CLONE_SYSVSEM only.
         let sysname = "unshare";
         match ScmpSyscall::from_name(sysname) {
             Ok(syscall) => {
                 let flags = if is_crypt {
-                    libc::CLONE_FS as u64
+                    (libc::CLONE_FS | libc::CLONE_SYSVSEM) as u64
                 } else {
-                    (libc::CLONE_FILES | libc::CLONE_FS) as u64
+                    (libc::CLONE_FILES | libc::CLONE_FS | libc::CLONE_SYSVSEM) as u64
                 };
                 ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg0 == flags)])?;
             }
