@@ -319,6 +319,13 @@ impl WorkerCache {
         Ok(())
     }
 
+    pub(crate) fn retire_unix_map(&self, pid: Pid) {
+        self.unix_map
+            .write()
+            .unwrap_or_else(|err| err.into_inner())
+            .retain(|_, val| val.pid != pid);
+    }
+
     // Remove a PID completely from the cache.
     pub(crate) fn del_pid(&self, pid: Pid) {
         // Retire TGID from signal maps.
@@ -326,6 +333,9 @@ impl WorkerCache {
         self.retire_sig_restart(pid);
         self.retire_ptrace(pid);
 
+        // Remove unix inode records for pid.
+        self.retire_unix_map(pid);
+
         // Remove preexisting error record for pid.
         let _ = self.get_error(pid);
 
